#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
/* #include "ie.h" */

#define DOUBLE_RAND ( (double)rand() / (double)(RAND_MAX-1) )

double rtclock() { 
  struct timezone Tzp; 
  struct timeval Tp; 
  int stat; 
  stat = gettimeofday (&Tp, &Tzp); 
  if (stat != 0) printf("Error return from gettimeofday: %d",stat); 
  return(Tp.tv_sec + Tp.tv_usec*1.0e-6); 
}


void print_output(double * phi, double * rho, double * phi_old, int nboxes, int * size_y, int * size_x)
{
  FILE* outfile = fopen("orig_output.dat","w");
  fprintf(outfile,"Nboxes : %d\n",nboxes);
  int i,j,k;
  int box_offset = 0;
  for( i = 0 ; i < nboxes; i++ ){
    fprintf(outfile,"Box : %d, size_y : %d, size_x : %d\n",i,size_y[i],size_x[i]);
    fprintf(outfile,"\tPhi :\n");
    for( j = 0 ; j < size_y[i] ; j++ ){
      fprintf(outfile,"\t");
      for( k = 0 ; k < size_x[i] ; k++ )
	fprintf(outfile," %lf",phi[box_offset + j*size_x[i] + k]);
      fprintf(outfile,"\n");
    }
    fprintf(outfile,"\tPhi_old :\n");
    for (j = 0; j < size_y[i]; j++) {
      fprintf(outfile,"\t");
      for (k = 0; k < size_x[i]; k++) 
        fprintf(outfile," %lf",phi_old[box_offset+j*size_x[i]+k]);
      fprintf(outfile,"\n");
    }
    fprintf(outfile,"\tRho :\n");
    for( j = 0 ; j < size_y[i] ; j++ ){
      fprintf(outfile,"\t");
      for( k = 0 ; k < size_x[i] ; k++ )
	fprintf(outfile," %lf",rho[box_offset+j*size_x[i]+k]);
      fprintf(outfile,"\n");
    }
    box_offset += size_y[i] * size_x[i];
  }
  fclose(outfile);
}


int main(int argc, char** argv){
  if( argc != 3 ){
    printf("Usage : ./%s <N> <ncroot>\n",argv[0]);
    exit(1);
  }

  int i,j,k,p,t;

  int N = atoi(argv[1]);
  int ncroot=atoi(argv[2]); ///Number of coarse grid boxes
  int nc = ncroot*ncroot;
  int nf=2*nc;    //Number of find grid boxes
  int nbdys = 4;
  double * bdy_value = (double*) malloc(sizeof(double)*nbdys);
  bdy_value[0] = DOUBLE_RAND;
  bdy_value[1] = DOUBLE_RAND;
  bdy_value[2] = DOUBLE_RAND;
  bdy_value[3] = DOUBLE_RAND;  

  int nboxes = nf+nc;

  int * coarse_boxes = (int*) malloc(sizeof(int)*nc); ///The coarse boxes in the list
  int * fine_boxes = (int*) malloc(sizeof(int)*nf); /// The fine boxes in the list
  
  for( i = 0 ; i < nc ; i++ )
    coarse_boxes[i] = i;
  for( i = 0 ; i < nf ; i++) 
    fine_boxes[i] = nc+i;

  int * start_y = (int*) malloc(sizeof(int)*nboxes); ///Start location of each box along y
  int * end_y = (int*) malloc(sizeof(int)*nboxes); ///End location of each box along y
  int * start_x = (int*) malloc(sizeof(int)*nboxes); ///Start location of each box along x
  int * end_x = (int*) malloc(sizeof(int)*nboxes); ///End location of each box along x
  int * size_y = (int*) malloc(sizeof(int)*nboxes);
  int * size_x = (int*) malloc(sizeof(int)*nboxes);

  for( i = 0 ; i < ncroot ; i++ )
    for( j = 0 ; j < ncroot ; j++ ){
      start_y[i*ncroot+j] = (N / ncroot) * i;
      start_x[i*ncroot+j] = (N / ncroot) * j;
      end_y[i*ncroot+j] = ( i == ncroot -1 ? N : (N / ncroot )*(i+1));
      end_x[i*ncroot+j] = ( j == ncroot -1 ? N : (N / ncroot )*(j+1));
      size_y[i*ncroot+j] = end_y[i*ncroot+j] - start_y[i*ncroot+j];
      size_x[i*ncroot+j] = end_x[i*ncroot+j] - start_x[i*ncroot+j];
    }

  int * ftoc = (int*) malloc(sizeof(int)*nf);
  
  for( i = 0 ; i < nc ; i++){
    ftoc[2*i] = i;
    ftoc[2*i+1] = i;
    start_x[nc+2*i] = start_x[i]+1;
    end_x[nc+2*i] = ( end_x[i] - 1 - start_x[i] ) / 2.0 + start_x[nc+2*i];
    start_x[nc+2*i+1] = end_x[nc+2*i];
    end_x[nc+2*i+1] = end_x[i]-1;
    size_x[nc+2*i] = (end_x[nc+2*i] - start_x[nc+2*i])*2;
    size_x[nc+2*i+1] = (end_x[nc+2*i+1] - start_x[nc+2*i+1])*2;
    start_y[nc+2*i] = start_y[i]+1;
    end_y[nc+2*i] = end_y[i]-1;
    start_y[nc+2*i+1] = start_y[nc+2*i];
    end_y[nc+2*i+1] = end_y[nc+2*i];
    size_y[nc+2*i] = (end_y[nc+2*i] - start_y[nc+2*i])*2;
    size_y[nc+2*i+1] = (end_y[nc+2*i+1] - start_y[nc+2*i+1])*2;
  }
  

  int max_c_y = 0;
  int max_c_x = 0;
  for( k = 0 ; k < nc ; k++ ){
    max_c_x = ( size_x[coarse_boxes[k]] > max_c_x ? size_x[coarse_boxes[k]] : max_c_x );
    max_c_y = ( size_y[coarse_boxes[k]] > max_c_y ? size_y[coarse_boxes[k]] : max_c_y );
  }
  double phi_c_t[max_c_y+2][max_c_x+2];

  int max_f_y = 0;
  int max_f_x = 0;
  for( k = 0 ; k < nf ; k++ ){
    max_f_x = ( size_x[fine_boxes[k]] > max_f_x ? size_x[fine_boxes[k]] : max_f_x );
    max_f_y = ( size_y[fine_boxes[k]] > max_f_y ? size_y[fine_boxes[k]] : max_f_y );
  }
  double phi_f_t[max_f_y+4][max_f_x+4];
  double residual[max_f_y][max_f_x];

  int phi_size = 0;
  int * box_offset = (int*)malloc(sizeof(int)*nboxes);
  for(k = 0; k < nboxes ; k++ ){
    box_offset[k] = phi_size;
    phi_size += size_x[k] * size_y[k];
  }
  double * phi = (double*)malloc(sizeof(double)*phi_size);
  double * phi_old = (double*)malloc(sizeof(double)*phi_size);
  double * rho = (double*)malloc(sizeof(double)*phi_size);
  for( i = 0 ; i < phi_size ; i++ ){
    phi[i] = DOUBLE_RAND;
    phi_old[i] = DOUBLE_RAND;
    rho[i] = DOUBLE_RAND;
  }
          
  int * patches = (int*) malloc(sizeof(int)*(nboxes+1)); ///The number of boundary patches for each box, stored as a prefix sum;
  for( i = 0 ; i < nboxes+1 ; i++ )
    patches[i] = i*4;
  
  int * patch_start_y = (int*) malloc(sizeof(int)*patches[nboxes]); ///The start position of the boundary patch along y, local to each box;
  int * patch_end_y = (int*) malloc(sizeof(int)*patches[nboxes]); ///The end position of the boundary patch along y, local to each box;
  int * patch_start_x = (int*) malloc(sizeof(int)*patches[nboxes]); ///The start position of the boundary patch along x, local to each box;
  int * patch_end_x = (int*) malloc(sizeof(int)*patches[nboxes]); ///The end position of the boundary patch along x, local to each box;
  int * neighb = (int*) malloc(sizeof(int)*patches[nboxes]);
  int * neighb_level = (int*) malloc(sizeof(int)*patches[nboxes]);

  for( p = 0 ; p < nc ; p++ ){
    patch_start_y[4*p] = start_y[p]-1;
    patch_end_y[4*p] = start_y[p];
    patch_start_x[4*p] =  start_x[p] ;
    patch_end_x[4*p] =  end_x[p];

    patch_start_y[4*p+1] = start_y[p];
    patch_end_y[4*p+1] = end_y[p];
    patch_start_x[4*p+1] =  end_x[p] ;
    patch_end_x[4*p+1] =  end_x[p]+1;
    
    patch_start_y[4*p+2] = end_y[p];
    patch_end_y[4*p+2] = end_y[p]+1;
    patch_start_x[4*p+2] =  start_x[p] ;
    patch_end_x[4*p+2] =  end_x[p];

    patch_start_y[4*p+3] = start_y[p];
    patch_end_y[4*p+3] = end_y[p];
    patch_start_x[4*p+3] =  start_x[p]-1;
    patch_end_x[4*p+3] =  start_x[p];
    
    if( p/ncroot == 0 ){
      neighb_level[4*p] = -1;
      neighb[4*p] = 0;
    }
    else{
      neighb_level[4*p] = 0;
      neighb[4*p] = p - ncroot ;
    }

    if( (p+1)%ncroot == 0 ){
      neighb_level[4*p+1] = -1;
      neighb[4*p+1] = 1;
    }
    else{
      neighb_level[4*p+1] = 0;
      neighb[4*p+1] = p + 1 ;
    }

    if( (p)%ncroot == 0 ){
      neighb_level[4*p+3] = -1;
      neighb[4*p+3] = 3;
    }
    else{
      neighb_level[4*p+3] = 0;
      neighb[4*p+3] = p-1;
    }

    if( (p)/ncroot == ncroot - 1 ){
      neighb_level[4*p+2] = -1;
      neighb[4*p+2] = 2;
    }
    else{
      neighb_level[4*p+2] = 0;
      neighb[4*p+2] = p+ncroot;
    }
  }

  for( p = nc ; p < nboxes ; p++ ){
    patch_start_y[4*p] = start_y[p]-1;
    patch_end_y[4*p] = start_y[p];
    patch_start_x[4*p] =  start_x[p] ;
    patch_end_x[4*p] =  end_x[p];

    patch_start_y[4*p+1] = start_y[p];
    patch_end_y[4*p+1] = end_y[p];
    patch_start_x[4*p+1] =  end_x[p] ;
    patch_end_x[4*p+1] =  end_x[p]+1;
    
    patch_start_y[4*p+2] = end_y[p];
    patch_end_y[4*p+2] = end_y[p]+1;
    patch_start_x[4*p+2] =  start_x[p] ;
    patch_end_x[4*p+2] =  end_x[p];

    patch_start_y[4*p+3] = start_y[p];
    patch_end_y[4*p+3] = end_y[p];
    patch_start_x[4*p+3] =  start_x[p]-1;
    patch_end_x[4*p+3] =  start_x[p];
  }
  
  for( p = nc ; p < nboxes ; p+=2 ){
    neighb_level[4*p] = 0 ; neighb[4*p] = (p-nc)/2;
    neighb_level[4*p+1] = 1; neighb[4*p+1] = fine_boxes[p-nc]+1;
    neighb_level[4*p+2] = 0 ; neighb[4*p+2] = (p-nc)/2;
    neighb_level[4*p+3] = 0 ; neighb[4*p+3] = (p-nc)/2;
    
    neighb_level[4*(p+1)] = 0 ; neighb[4*(p+1)] = (p-nc)/2;
    neighb_level[4*(p+1)+1] = 0; neighb[4*(p+1)+1] = (p-nc)/2;
    neighb_level[4*(p+1)+2] = 0 ; neighb[4*(p+1)+2] = (p-nc)/2;
    neighb_level[4*(p+1)+3] = 1 ; neighb[4*(p+1)+3] = fine_boxes[(p-nc)+1]-1;;
  }
  double mimic_temp;
  double read_temp = 0.0;
  double * rho_write;
  int patch_size = patches[nboxes];

#pragma arrays fine_boxes[nf][1],size_y[nboxes][1],size_x[nboxes][1],patches[nboxes+1][1],patch_start_y[patch_size][1],patch_start_x[patch_size][1],patch_end_y[patch_size][1],patch_end_x[patch_size][1],neighb_level[patch_size][1],neighb[patch_size][1],box_offset[nboxes][1],start_y[nboxes][1],start_x[nboxes][1],end_y[nboxes][1],end_x[nboxes][1],ftoc[nf][1],phi[phi_size][1],phi_old[phi_size][1],rho[phi_size][1],rho_write[phi_size][1]

#pragma orig_loop
  for( t = 0 ; t < 10 ; t++ ){

#pragma part_loop
    for( k = 0 ; k < nf ; k++ ){
      for( i = 0 ; i < size_y[fine_boxes[k]] ; i++ )
	for( j = 0 ; j < size_x[fine_boxes[k]] ; j++ )
	  phi_f_t[i+2][j+2] = phi_old[box_offset[fine_boxes[k]]+ i*size_x[fine_boxes[k]] + j];

      ///Initializing the boundaries
      for( p = patches[fine_boxes[k]] ; p < patches[fine_boxes[k]+1] ; p++ ){
	if( neighb_level[p] == 1 )
	  for( i = patch_start_y[p] ; i < patch_end_y[p] ; i++ )     /// 2,8
	    for( j = patch_start_x[p] ; j < patch_end_x[p] ; j++ ) { /// 8,10
	      phi_f_t[i*2+2-start_y[fine_boxes[k]]*2][j*2+2-start_x[fine_boxes[k]]*2] = phi_old[box_offset[neighb[p]] + (i*2-start_y[neighb[p]]*2)*size_x[neighb[p]] + j*2-start_x[neighb[p]]*2]; /// 2,10 ; 2,6
	      phi_f_t[i*2+3-start_y[fine_boxes[k]]*2][j*2+2-start_x[fine_boxes[k]]*2] = phi_old[box_offset[neighb[p]] + (i*2+1-start_y[neighb[p]]*2)*size_x[neighb[p]] + j*2-start_x[neighb[p]]*2]; /// 2,10 ; 2,6
	      phi_f_t[i*2+2-start_y[fine_boxes[k]]*2][j*2+3-start_x[fine_boxes[k]]*2] = phi_old[box_offset[neighb[p]] + (i*2-start_y[neighb[p]]*2)*size_x[neighb[p]] + j*2+1-start_x[neighb[p]]*2]; /// 2,10 ; 2,6
	      phi_f_t[i*2+3-start_y[fine_boxes[k]]*2][j*2+3-start_x[fine_boxes[k]]*2] = phi_old[box_offset[neighb[p]] + (i*2+1-start_y[neighb[p]]*2)*size_x[neighb[p]] + j*2+1-start_x[neighb[p]]*2]; /// 2,10 ; 2,6
	    }
	else
	  for( i = patch_start_y[p] ; i < patch_end_y[p] ; i++ )
	    for( j = patch_start_x[p] ; j < patch_end_x[p] ; j++ ){
	      phi_f_t[2*i+2-start_y[fine_boxes[k]]*2][2*j+2-start_x[fine_boxes[k]]*2] = phi_old[box_offset[neighb[p]]+ (i-start_y[neighb[p]])*size_x[neighb[p]] + j-start_x[neighb[p]]];
	      phi_f_t[2*i+3-start_y[fine_boxes[k]]*2][2*j+2-start_x[fine_boxes[k]]*2] = phi_old[box_offset[neighb[p]]+ (i-start_y[neighb[p]])*size_x[neighb[p]] + j-start_x[neighb[p]]];
	      phi_f_t[2*i+2-start_y[fine_boxes[k]]*2][2*j+3-start_x[fine_boxes[k]]*2] = phi_old[box_offset[neighb[p]]+ (i-start_y[neighb[p]])*size_x[neighb[p]] + j-start_x[neighb[p]]];
	      phi_f_t[2*i+3-start_y[fine_boxes[k]]*2][2*j+3-start_x[fine_boxes[k]]*2] = phi_old[box_offset[neighb[p]]+ (i-start_y[neighb[p]])*size_x[neighb[p]] + j-start_x[neighb[p]]];
	    }
      }

      for( i = 0 ; i < size_y[fine_boxes[k]] ; i++ )
	for( j = 0 ; j < size_x[fine_boxes[k]] ; j++ )
	  phi[box_offset[fine_boxes[k]] + i*size_x[fine_boxes[k]] + j] = ( phi_f_t[i+1][j+2] + phi_f_t[i+2][j+1] + phi_f_t[i+2][j+3] + phi_f_t[i+3][j+2] - 4 * phi_f_t[i+2][j+2]  - rho[box_offset[fine_boxes[k]]+i*size_x[fine_boxes[k]]+j] * 0.25 ) / 8.0;
    }


#pragma part_loop
    for( k = 0 ; k < nf ; k++ ){
      for( i = 0 ; i < size_y[fine_boxes[k]] ; i++ )
	for( j = 0 ; j < size_x[fine_boxes[k]] ; j++ )
	  phi_f_t[i+2][j+2] = phi[box_offset[fine_boxes[k]] + i* size_x[fine_boxes[k]] + j];
	  /* mimic_temp = phi[box_offset[fine_boxes[k]] + i* size_x[fine_boxes[k]] + j]; */

      ///Initializing the boundaries
      for( p = patches[fine_boxes[k]] ; p < patches[fine_boxes[k]+1] ; p++ ){
	if( neighb_level[p] == 1 )
	  for( i = patch_start_y[p] ; i < patch_end_y[p] ; i++ )     /// 2,8
	    for( j = patch_start_x[p] ; j < patch_end_x[p] ; j++ ) { /// 8,10
	      phi_f_t[i*2+2-start_y[fine_boxes[k]]*2][j*2+2-start_x[fine_boxes[k]]*2] = phi[box_offset[neighb[p]] + (i*2-start_y[neighb[p]]*2)*size_x[neighb[p]] + j*2-start_x[neighb[p]]*2]; /// 2,10 ; 2,6
	      /* mimic_temp = phi[box_offset[neighb[p]] + (i*2-start_y[neighb[p]]*2)*size_x[neighb[p]] + j*2-start_x[neighb[p]]*2]; */
	      phi_f_t[i*2+3-start_y[fine_boxes[k]]*2][j*2+2-start_x[fine_boxes[k]]*2] = phi[box_offset[neighb[p]] + (i*2+1-start_y[neighb[p]]*2)*size_x[neighb[p]] + j*2-start_x[neighb[p]]*2]; /// 2,10 ; 2,6
	      /* mimic_temp = phi[box_offset[neighb[p]] + (i*2-start_y[neighb[p]]*2)*size_x[neighb[p]] + j*2-start_x[neighb[p]]*2]; // */
	      phi_f_t[i*2+2-start_y[fine_boxes[k]]*2][j*2+3-start_x[fine_boxes[k]]*2] = phi[box_offset[neighb[p]] + (i*2-start_y[neighb[p]]*2)*size_x[neighb[p]] + j*2+1-start_x[neighb[p]]*2]; /// 2,10 ; 2,6
	      /* mimic_temp = phi[box_offset[neighb[p]] + (i*2-start_y[neighb[p]]*2)*size_x[neighb[p]] + j*2-start_x[neighb[p]]*2]; // */
	      phi_f_t[i*2+3-start_y[fine_boxes[k]]*2][j*2+3-start_x[fine_boxes[k]]*2] = phi[box_offset[neighb[p]] + (i*2+1-start_y[neighb[p]]*2)*size_x[neighb[p]] + j*2+1-start_x[neighb[p]]*2]; /// 2,10 ; 2,6
	      /* mimic_temp = phi[box_offset[neighb[p]] + (i*2-start_y[neighb[p]]*2)*size_x[neighb[p]] + j*2-start_x[neighb[p]]*2]; // */
	    }
	else
	  for( i = patch_start_y[p] ; i < patch_end_y[p] ; i++ )
	    for( j = patch_start_x[p] ; j < patch_end_x[p] ; j++ ){
	      phi_f_t[2*i+2-start_y[fine_boxes[k]]*2][2*j+2-start_x[fine_boxes[k]]*2] = phi[box_offset[neighb[p]] + (i-start_y[neighb[p]])*size_x[neighb[p]] + j - start_x[neighb[p]]];
	      /* mimic_temp = phi[box_offset[neighb[p]] + (i-start_y[neighb[p]])*size_x[neighb[p]] + j - start_x[neighb[p]]]; */
	      phi_f_t[2*i+3-start_y[fine_boxes[k]]*2][2*j+2-start_x[fine_boxes[k]]*2] = phi[box_offset[neighb[p]] + (i-start_y[neighb[p]])*size_x[neighb[p]] + j - start_x[neighb[p]]];
	      /* mimic_temp = phi[box_offset[neighb[p]] + (i-start_y[neighb[p]])*size_x[neighb[p]] + j - start_x[neighb[p]]]; */
	      phi_f_t[2*i+2-start_y[fine_boxes[k]]*2][2*j+3-start_x[fine_boxes[k]]*2] = phi[box_offset[neighb[p]] + (i-start_y[neighb[p]])*size_x[neighb[p]] + j - start_x[neighb[p]]];
	      /* mimic_temp = phi[box_offset[neighb[p]] + (i-start_y[neighb[p]])*size_x[neighb[p]] + j - start_x[neighb[p]]]; */
	      phi_f_t[2*i+3-start_y[fine_boxes[k]]*2][2*j+3-start_x[fine_boxes[k]]*2] = phi[box_offset[neighb[p]] + (i-start_y[neighb[p]])*size_x[neighb[p]] + j - start_x[neighb[p]]];
	      /* mimic_temp = phi[box_offset[neighb[p]] + (i-start_y[neighb[p]])*size_x[neighb[p]] + j - start_x[neighb[p]]]; */
	    }
      }

      for( i = 0 ; i < size_y[fine_boxes[k]] ; i++ )
	for( j = 0 ; j < size_x[fine_boxes[k]] ; j++ )
	  residual[i][j] = rho[box_offset[fine_boxes[k]] + i*size_x[fine_boxes[k]] + j] - ( phi_f_t[i+1][j+2] + phi_f_t[i+2][j+1] + phi_f_t[i+2][j+3] + phi_f_t[i+3][j+2] - 4 * phi_f_t[i+2][j+2] ) * 4.0;
	  /* mimic_temp = rho[box_offset[fine_boxes[k]] + i*size_x[fine_boxes[k]] + j]; */
  
      for( i = start_y[fine_boxes[k]] ; i < end_y[fine_boxes[k]] ; i++ )
	for( j = start_x[fine_boxes[k]] ; j < end_x[fine_boxes[k]]; j++ ){
	  rho[box_offset[ftoc[k]] + (i-start_y[ftoc[k]])*size_x[ftoc[k]] + (j-start_x[ftoc[k]])] = ( residual[2*(i-start_y[fine_boxes[k]])][2*(j-start_x[fine_boxes[k]])] + residual[2*(i-start_y[fine_boxes[k]])+1][2*(j-start_x[fine_boxes[k]])] + residual[2*(i-start_y[fine_boxes[k]])][2*(j-start_x[fine_boxes[k]])+1] + residual[2*(i-start_y[fine_boxes[k]])+1][2*(j-start_x[fine_boxes[k]])+1] ) / 4.0;
	  /* rho_write[box_offset[ftoc[k]] + (i-start_y[ftoc[k]])*size_x[ftoc[k]] + (j-start_x[ftoc[k]])] = read_temp; */
	  /* phi_old[ftoc[k]][i-start_y[ftoc[k]]][j-start_x[ftoc[k]]] = 0.0; */
	  phi_old[box_offset[ftoc[k]] + (i-start_y[ftoc[k]])*size_x[ftoc[k]] + (j-start_x[ftoc[k]])] = 0.0;
	}
    }


#pragma part_loop
    ///phi_c += alpha * ( - rho_c )
    for( k = 0 ; k < nc ; k++ ){

      for( i = 0 ; i < size_y[coarse_boxes[k]] ; i++ )
    	for( j = 0 ; j < size_x[coarse_boxes[k]] ; j++ )
    	  phi_c_t[i+1][j+1] = phi_old[box_offset[coarse_boxes[k]] + i*size_x[coarse_boxes[k]] + j];
    
      for( p = patches[coarse_boxes[k]] ; p < patches[coarse_boxes[k]+1]; p++)
      	if( neighb_level[p] == -1 ){
      	  for( i = patch_start_y[p] ; i < patch_end_y[p] ; i++ )   /// 5 , 6
      	    for( j = patch_start_x[p] ; j < patch_end_x[p] ; j++ ) /// 5 , 10
      	      phi_c_t[i+1-start_y[coarse_boxes[k]]][j+1-start_x[coarse_boxes[k]]] = bdy_value[neighb[p]]; /// 0 , 5
      	}
      	else{
      	  for( i = patch_start_y[p] ; i < patch_end_y[p] ; i++ )    /// 0,5
      	    for( j = patch_start_x[p] ; j < patch_end_x[p] ; j++ )  /// 4,5
      	      phi_c_t[i+1-start_y[coarse_boxes[k]]][j+1-start_x[coarse_boxes[k]]] = phi_old[box_offset[neighb[p]] + (i - start_y[neighb[p]])*size_x[neighb[p]] + j - start_x[neighb[p]]]; /// 0,5 ; 0,0
      	}
      for( i = 0 ; i < size_y[coarse_boxes[k]] ; i++ )
      	for( j = 0 ; j < size_x[coarse_boxes[k]] ; j++ )
      	  phi[box_offset[coarse_boxes[k]] + i*size_x[coarse_boxes[k]] + j] =  ( phi_c_t[i][j+1] + phi_c_t[i+1][j] + phi_c_t[i+1][j+2] + phi_c_t[i+2][j+1] - 4 * phi_c_t[i+1][j+1] - rho[box_offset[coarse_boxes[k]] + i*size_x[coarse_boxes[k]] + j] ) / 8.0;
    }

    ///phi_f += P_cf(phi_c)
#pragma part_loop
    for( k = 0 ; k < nf ; k++) {
      for( i = start_y[fine_boxes[k]] ; i < end_y[fine_boxes[k]] ; i++ )
    	for( j = start_x[fine_boxes[k]] ; j < end_x[fine_boxes[k]]; j++ ){
    	  phi_old[box_offset[fine_boxes[k]] + (i-start_y[fine_boxes[k]])*2*size_x[fine_boxes[k]] + (j-start_x[fine_boxes[k]])*2] = phi[box_offset[ftoc[k]]+ (i-start_y[ftoc[k]])*size_x[ftoc[k]] + j-start_x[ftoc[k]]];
    	  phi_old[box_offset[fine_boxes[k]] + ((i-start_y[fine_boxes[k]])*2+1)*size_x[fine_boxes[k]] + (j-start_x[fine_boxes[k]])*2] = phi[box_offset[ftoc[k]]+ (i-start_y[ftoc[k]])*size_x[ftoc[k]] + j-start_x[ftoc[k]]];
    	  phi_old[box_offset[fine_boxes[k]] + (i-start_y[fine_boxes[k]])*2*size_x[fine_boxes[k]] + (j-start_x[fine_boxes[k]])*2+1] = phi[box_offset[ftoc[k]]+ (i-start_y[ftoc[k]])*size_x[ftoc[k]] + j-start_x[ftoc[k]]];
	  phi_old[box_offset[fine_boxes[k]] + ((i-start_y[fine_boxes[k]])*2+1)*size_x[fine_boxes[k]] + (j-start_x[fine_boxes[k]])*2+1] = phi[box_offset[ftoc[k]]+ (i-start_y[ftoc[k]])*size_x[ftoc[k]] + j-start_x[ftoc[k]]];
    	}
    }

  }
  print_output(phi,rho,phi_old,nboxes,size_y,size_x);

  free(bdy_value);
  free(coarse_boxes);
  free(fine_boxes);
  free(start_y);
  free(end_y);
  free(start_x);
  free(end_x);
  free(size_y);
  free(size_x);
  free(ftoc);
  free(patches);
  free(patch_start_y);
  free(patch_end_y);
  free(patch_start_x);
  free(patch_end_x);
  free(neighb_level);
  free(neighb);
  free(box_offset);
  free(phi);
  free(phi_old);
  free(rho);
}
