#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include "ie.h"
#define DOUBLE_RAND ( (double)rand() / (double)(RAND_MAX-1) )

double rtclock()
{
  struct timezone Tzp;
  struct timeval Tp;
  int stat;
  stat = gettimeofday(&Tp,&Tzp);
  if (stat != 0) 
    printf("Error return from gettimeofday: %d",stat);
  return Tp.tv_sec + (Tp.tv_usec * 1.0e-6);
}

void print_output(double * phi, double * rho, double * phi_old, int nboxes, int * size_y, int * size_x)
{
  FILE* outfile = fopen("rose_output.dat","w");
  fprintf(outfile,"Nboxes : %d\n",nboxes);
  int i,j,k;
  int box_offset = 0;
  for( i = 0 ; i < nboxes; i++ ){
    fprintf(outfile,"Box : %d, size_y : %d, size_x : %d\n",i,size_y[i],size_x[i]);
    fprintf(outfile,"\tPhi :\n");
    for( j = 0 ; j < size_y[i] ; j++ ){
      fprintf(outfile,"\t");
      for( k = 0 ; k < size_x[i] ; k++ )
	fprintf(outfile," %lf",phi[box_offset + j*size_x[i] + k]);
      fprintf(outfile,"\n");
    }
    fprintf(outfile,"\tPhi_old :\n");
    for (j = 0; j < size_y[i]; j++) {
      fprintf(outfile,"\t");
      for (k = 0; k < size_x[i]; k++) 
        fprintf(outfile," %lf",phi_old[box_offset+j*size_x[i]+k]);
      fprintf(outfile,"\n");
    }
    fprintf(outfile,"\tRho :\n");
    for( j = 0 ; j < size_y[i] ; j++ ){
      fprintf(outfile,"\t");
      for( k = 0 ; k < size_x[i] ; k++ )
	fprintf(outfile," %lf",rho[box_offset+j*size_x[i]+k]);
      fprintf(outfile,"\n");
    }
    box_offset += size_y[i] * size_x[i];
  }
  fclose(outfile);
}

int main(int argc,char **argv)
{
  MPI_Init(&argc,&argv);
  ARMCI_Init();
  MPI_Comm_rank(MPI_COMM_WORLD,&__proc_id__);
  MPI_Comm_size(MPI_COMM_WORLD,&__nprocs__);
  if (argc != 3) {
    printf("Usage : ./%s <N> <ncroot>\n",argv[0]);
    exit(1);
  }
  int i;
  int j;
  int k;
  int p;
  int t;
  int N = atoi(argv[1]);
///Number of coarse grid boxes
  int ncroot = atoi(argv[2]);
  int nc = (ncroot * ncroot);
//Number of find grid boxes
  int nf = (2 * nc);
  int nbdys = 4;
  double *bdy_value = (double *)(malloc((sizeof(double ) * nbdys)));
  bdy_value[0] = (((double )(rand())) / ((double )(2147483647 - 1)));
  bdy_value[1] = (((double )(rand())) / ((double )(2147483647 - 1)));
  bdy_value[2] = (((double )(rand())) / ((double )(2147483647 - 1)));
  bdy_value[3] = (((double )(rand())) / ((double )(2147483647 - 1)));
  int nboxes = (nf + nc);
///The coarse boxes in the list
  int *coarse_boxes = (int *)(malloc((sizeof(int ) * nc)));
/// The fine boxes in the list
  int *fine_boxes = (int *)(malloc((sizeof(int ) * nf)));
  for (i = 0; i < nc; i++) 
    coarse_boxes[i] = i;
  for (i = 0; i < nf; i++) 
    fine_boxes[i] = (nc + i);
///Start location of each box along y
  int *start_y = (int *)(malloc((sizeof(int ) * nboxes)));
///End location of each box along y
  int *end_y = (int *)(malloc((sizeof(int ) * nboxes)));
///Start location of each box along x
  int *start_x = (int *)(malloc((sizeof(int ) * nboxes)));
///End location of each box along x
  int *end_x = (int *)(malloc((sizeof(int ) * nboxes)));
  int *size_y = (int *)(malloc((sizeof(int ) * nboxes)));
  int *size_x = (int *)(malloc((sizeof(int ) * nboxes)));
  for (i = 0; i < ncroot; i++) 
    for (j = 0; j < ncroot; j++) {
      start_y[(i * ncroot) + j] = ((N / ncroot) * i);
      start_x[(i * ncroot) + j] = ((N / ncroot) * j);
      end_y[(i * ncroot) + j] = ((i == (ncroot - 1))?N : ((N / ncroot) * (i + 1)));
      end_x[(i * ncroot) + j] = ((j == (ncroot - 1))?N : ((N / ncroot) * (j + 1)));
      size_y[(i * ncroot) + j] = (end_y[(i * ncroot) + j] - start_y[(i * ncroot) + j]);
      size_x[(i * ncroot) + j] = (end_x[(i * ncroot) + j] - start_x[(i * ncroot) + j]);
    }
  int *ftoc = (int *)(malloc((sizeof(int ) * nf)));
  for (i = 0; i < nc; i++) {
    ftoc[2 * i] = i;
    ftoc[(2 * i) + 1] = i;
    start_x[nc + (2 * i)] = (start_x[i] + 1);
    end_x[nc + (2 * i)] = ((((end_x[i] - 1) - start_x[i]) / 2.0) + start_x[nc + (2 * i)]);
    start_x[(nc + (2 * i)) + 1] = end_x[nc + (2 * i)];
    end_x[(nc + (2 * i)) + 1] = (end_x[i] - 1);
    size_x[nc + (2 * i)] = ((end_x[nc + (2 * i)] - start_x[nc + (2 * i)]) * 2);
    size_x[(nc + (2 * i)) + 1] = ((end_x[(nc + (2 * i)) + 1] - start_x[(nc + (2 * i)) + 1]) * 2);
    start_y[nc + (2 * i)] = (start_y[i] + 1);
    end_y[nc + (2 * i)] = (end_y[i] - 1);
    start_y[(nc + (2 * i)) + 1] = start_y[nc + (2 * i)];
    end_y[(nc + (2 * i)) + 1] = end_y[nc + (2 * i)];
    size_y[nc + (2 * i)] = ((end_y[nc + (2 * i)] - start_y[nc + (2 * i)]) * 2);
    size_y[(nc + (2 * i)) + 1] = ((end_y[(nc + (2 * i)) + 1] - start_y[(nc + (2 * i)) + 1]) * 2);
  }
  int max_c_y = 0;
  int max_c_x = 0;
  for (k = 0; k < nc; k++) {
    max_c_x = ((size_x[coarse_boxes[k]] > max_c_x)?size_x[coarse_boxes[k]] : max_c_x);
    max_c_y = ((size_y[coarse_boxes[k]] > max_c_y)?size_y[coarse_boxes[k]] : max_c_y);
  }
  int max_f_y = 0;
  int max_f_x = 0;
  for (k = 0; k < nf; k++) {
    max_f_x = ((size_x[fine_boxes[k]] > max_f_x)?size_x[fine_boxes[k]] : max_f_x);
    max_f_y = ((size_y[fine_boxes[k]] > max_f_y)?size_y[fine_boxes[k]] : max_f_y);
  }
  int phi_size = 0;
  int *box_offset = (int *)(malloc((sizeof(int ) * nboxes)));
  for (k = 0; k < nboxes; k++) {
    box_offset[k] = phi_size;
    phi_size += (size_x[k] * size_y[k]);
  }
  double *phi = (double *)(malloc((sizeof(double ) * phi_size)));
  double *phi_old = (double *)(malloc((sizeof(double ) * phi_size)));
  double *rho = (double *)(malloc((sizeof(double ) * phi_size)));
  for( i = 0 ; i < phi_size ; i++ ){
    phi[i] = DOUBLE_RAND;
    phi_old[i] = DOUBLE_RAND;
    rho[i] = DOUBLE_RAND;
  }

///The number of boundary patches for each box, stored as a prefix sum;
  int *patches = (int *)(malloc((sizeof(int ) * (nboxes + 1))));
  for (i = 0; i < (nboxes + 1); i++) 
    patches[i] = (i * 4);
///The start position of the boundary patch along y, local to each box;
  int *patch_start_y = (int *)(malloc((sizeof(int ) * patches[nboxes])));
///The end position of the boundary patch along y, local to each box;
  int *patch_end_y = (int *)(malloc((sizeof(int ) * patches[nboxes])));
///The start position of the boundary patch along x, local to each box;
  int *patch_start_x = (int *)(malloc((sizeof(int ) * patches[nboxes])));
///The end position of the boundary patch along x, local to each box;
  int *patch_end_x = (int *)(malloc((sizeof(int ) * patches[nboxes])));
  int *neighb = (int *)(malloc((sizeof(int ) * patches[nboxes])));
  int *neighb_level = (int *)(malloc((sizeof(int ) * patches[nboxes])));
  for (p = 0; p < nc; p++) {
    patch_start_y[4 * p] = (start_y[p] - 1);
    patch_end_y[4 * p] = start_y[p];
    patch_start_x[4 * p] = start_x[p];
    patch_end_x[4 * p] = end_x[p];
    patch_start_y[(4 * p) + 1] = start_y[p];
    patch_end_y[(4 * p) + 1] = end_y[p];
    patch_start_x[(4 * p) + 1] = end_x[p];
    patch_end_x[(4 * p) + 1] = (end_x[p] + 1);
    patch_start_y[(4 * p) + 2] = end_y[p];
    patch_end_y[(4 * p) + 2] = (end_y[p] + 1);
    patch_start_x[(4 * p) + 2] = start_x[p];
    patch_end_x[(4 * p) + 2] = end_x[p];
    patch_start_y[(4 * p) + 3] = start_y[p];
    patch_end_y[(4 * p) + 3] = end_y[p];
    patch_start_x[(4 * p) + 3] = (start_x[p] - 1);
    patch_end_x[(4 * p) + 3] = start_x[p];
    if ((p / ncroot) == 0) {
      neighb_level[4 * p] = -1;
      neighb[4 * p] = 0;
    }
    else {
      neighb_level[4 * p] = 0;
      neighb[4 * p] = (p - ncroot);
    }
    if (((p + 1) % ncroot) == 0) {
      neighb_level[(4 * p) + 1] = -1;
      neighb[(4 * p) + 1] = 1;
    }
    else {
      neighb_level[(4 * p) + 1] = 0;
      neighb[(4 * p) + 1] = (p + 1);
    }
    if ((p % ncroot) == 0) {
      neighb_level[(4 * p) + 3] = -1;
      neighb[(4 * p) + 3] = 3;
    }
    else {
      neighb_level[(4 * p) + 3] = 0;
      neighb[(4 * p) + 3] = (p - 1);
    }
    if ((p / ncroot) == (ncroot - 1)) {
      neighb_level[(4 * p) + 2] = -1;
      neighb[(4 * p) + 2] = 2;
    }
    else {
      neighb_level[(4 * p) + 2] = 0;
      neighb[(4 * p) + 2] = (p + ncroot);
    }
  }
  for (p = nc; p < nboxes; p++) {
    patch_start_y[4 * p] = (start_y[p] - 1);
    patch_end_y[4 * p] = start_y[p];
    patch_start_x[4 * p] = start_x[p];
    patch_end_x[4 * p] = end_x[p];
    patch_start_y[(4 * p) + 1] = start_y[p];
    patch_end_y[(4 * p) + 1] = end_y[p];
    patch_start_x[(4 * p) + 1] = end_x[p];
    patch_end_x[(4 * p) + 1] = (end_x[p] + 1);
    patch_start_y[(4 * p) + 2] = end_y[p];
    patch_end_y[(4 * p) + 2] = (end_y[p] + 1);
    patch_start_x[(4 * p) + 2] = start_x[p];
    patch_end_x[(4 * p) + 2] = end_x[p];
    patch_start_y[(4 * p) + 3] = start_y[p];
    patch_end_y[(4 * p) + 3] = end_y[p];
    patch_start_x[(4 * p) + 3] = (start_x[p] - 1);
    patch_end_x[(4 * p) + 3] = start_x[p];
  }
  for (p = nc; p < nboxes; p += 2) {
    neighb_level[4 * p] = 0;
    neighb[4 * p] = ((p - nc) / 2);
    neighb_level[(4 * p) + 1] = 1;
    neighb[(4 * p) + 1] = (fine_boxes[p - nc] + 1);
    neighb_level[(4 * p) + 2] = 0;
    neighb[(4 * p) + 2] = ((p - nc) / 2);
    neighb_level[(4 * p) + 3] = 0;
    neighb[(4 * p) + 3] = ((p - nc) / 2);
    neighb_level[4 * (p + 1)] = 0;
    neighb[4 * (p + 1)] = ((p - nc) / 2);
    neighb_level[(4 * (p + 1)) + 1] = 0;
    neighb[(4 * (p + 1)) + 1] = ((p - nc) / 2);
    neighb_level[(4 * (p + 1)) + 2] = 0;
    neighb[(4 * (p + 1)) + 2] = ((p - nc) / 2);
    neighb_level[(4 * (p + 1)) + 3] = 1;
    neighb[(4 * (p + 1)) + 3] = (fine_boxes[(p - nc) + 1] - 1);;
  }
  double mimic_temp;
  double mimic_temp2;
  double mimic_temp3;
  double mimic_temp4;
  double read_temp = 0.0;
  double read_temp2 = 0.0;
  double *rho_write;
  int patch_size = patches[nboxes];
  
#pragma arrays fine_boxes [ nf ] [ 1 ], size_y [ nboxes ] [ 1 ], size_x [ nboxes ] [ 1 ], patches [ nboxes + 1 ] [ 1 ], patch_start_y [ patch_size ] [ 1 ], patch_start_x [ patch_size ] [ 1 ], patch_end_y [ patch_size ] [ 1 ], patch_end_x [ patch_size ] [ 1 ], neighb_level [ patch_size ] [ 1 ], neighb [ patch_size ] [ 1 ], box_offset [ nboxes ] [ 1 ], start_y [ nboxes ] [ 1 ], start_x [ nboxes ] [ 1 ], end_y [ nboxes ] [ 1 ], end_x [ nboxes ] [ 1 ], ftoc [ nf ] [ 1 ], phi [ phi_size ] [ 1 ], phi_old [ phi_size ] [ 1 ], rho [ phi_size ] [ 1 ], rho_write [ phi_size ] [ 1 ], coarse_boxes [ nc ] [ 1 ]
  
#pragma orig_loop
  __nthreads__ = omp_get_max_threads();
  int iter_num_count[2] = {nf - 0, nc - 0};
  int data_num_count[3] = { phi_size , phi_size , phi_size };
  int ro_mask[3] = {0,0, 0};
  create_inspector(__proc_id__,__nprocs__,__nthreads__,2,3,4,17,iter_num_count,data_num_count,ro_mask);
  int __size_coarse_boxes__ =  nc ;
  set_access_array_param(0, nc , 1 ,__size_coarse_boxes__ / __nprocs__ * __proc_id__ + coarse_boxes);
  int __size_fine_boxes__ =  nf ;
  set_access_array_param(1, nf , 1 ,__size_fine_boxes__ / __nprocs__ * __proc_id__ + fine_boxes);
  int __size_start_y__ =  nboxes ;
  set_access_array_param(2, nboxes , 1 ,__size_start_y__ / __nprocs__ * __proc_id__ + start_y);
  int __size_end_y__ =  nboxes ;
  set_access_array_param(3, nboxes , 1 ,__size_end_y__ / __nprocs__ * __proc_id__ + end_y);
  int __size_start_x__ =  nboxes ;
  set_access_array_param(4, nboxes , 1 ,__size_start_x__ / __nprocs__ * __proc_id__ + start_x);
  int __size_end_x__ =  nboxes ;
  set_access_array_param(5, nboxes , 1 ,__size_end_x__ / __nprocs__ * __proc_id__ + end_x);
  int __size_size_y__ =  nboxes ;
  set_access_array_param(6, nboxes , 1 ,__size_size_y__ / __nprocs__ * __proc_id__ + size_y);
  int __size_size_x__ =  nboxes ;
  set_access_array_param(7, nboxes , 1 ,__size_size_x__ / __nprocs__ * __proc_id__ + size_x);
  int __size_ftoc__ =  nf ;
  set_access_array_param(8, nf , 1 ,__size_ftoc__ / __nprocs__ * __proc_id__ + ftoc);
  int __size_box_offset__ =  nboxes ;
  set_access_array_param(9, nboxes , 1 ,__size_box_offset__ / __nprocs__ * __proc_id__ + box_offset);
  int __size_patches__ =  nboxes + 1 ;
  set_access_array_param(10, nboxes + 1 , 1 ,__size_patches__ / __nprocs__ * __proc_id__ + patches);
  int __size_patch_start_y__ =  patch_size ;
  set_access_array_param(11, patch_size , 1 ,__size_patch_start_y__ / __nprocs__ * __proc_id__ + patch_start_y);
  int __size_patch_end_y__ =  patch_size ;
  set_access_array_param(12, patch_size , 1 ,__size_patch_end_y__ / __nprocs__ * __proc_id__ + patch_end_y);
  int __size_patch_start_x__ =  patch_size ;
  set_access_array_param(13, patch_size , 1 ,__size_patch_start_x__ / __nprocs__ * __proc_id__ + patch_start_x);
  int __size_patch_end_x__ =  patch_size ;
  set_access_array_param(14, patch_size , 1 ,__size_patch_end_x__ / __nprocs__ * __proc_id__ + patch_end_x);
  int __size_neighb__ =  patch_size ;
  set_access_array_param(15, patch_size , 1 ,__size_neighb__ / __nprocs__ * __proc_id__ + neighb);
  int __size_neighb_level__ =  patch_size ;
  set_access_array_param(16, patch_size , 1 ,__size_neighb_level__ / __nprocs__ * __proc_id__ + neighb_level);
  int __local_block_0_start__ = (nf - 0) / __nprocs__ * __proc_id__ + 0;
  int __local_block_0_end__;
  if (__proc_id__ != __nprocs__ - 1) 
    __local_block_0_end__ = (nf - 0) / __nprocs__ * (__proc_id__ + 1) + 0;
  else 
    __local_block_0_end__ = nf;
  int *__0_status__ = (int *)(malloc(sizeof(int ) * (__local_block_0_end__ - __local_block_0_start__)));
  memset(__0_status__,0,sizeof(int ) * (__local_block_0_end__ - __local_block_0_start__));
  int __local_block_1_start__ = (nf - 0) / __nprocs__ * __proc_id__ + 0;
  int __local_block_1_end__;
  if (__proc_id__ != __nprocs__ - 1) 
    __local_block_1_end__ = (nf - 0) / __nprocs__ * (__proc_id__ + 1) + 0;
  else 
    __local_block_1_end__ = nf;
  int *__1_status__ = (int *)(malloc(sizeof(int ) * (__local_block_1_end__ - __local_block_1_start__)));
  memset(__1_status__,0,sizeof(int ) * (__local_block_1_end__ - __local_block_1_start__));
  int __local_block_2_start__ = (nc - 0) / __nprocs__ * __proc_id__ + 0;
  int __local_block_2_end__;
  if (__proc_id__ != __nprocs__ - 1) 
    __local_block_2_end__ = (nc - 0) / __nprocs__ * (__proc_id__ + 1) + 0;
  else 
    __local_block_2_end__ = nc;
  int *__2_status__ = (int *)(malloc(sizeof(int ) * (__local_block_2_end__ - __local_block_2_start__)));
  memset(__2_status__,0,sizeof(int ) * (__local_block_2_end__ - __local_block_2_start__));
  int __local_block_3_start__ = (nf - 0) / __nprocs__ * __proc_id__ + 0;
  int __local_block_3_end__;
  if (__proc_id__ != __nprocs__ - 1) 
    __local_block_3_end__ = (nf - 0) / __nprocs__ * (__proc_id__ + 1) + 0;
  else 
    __local_block_3_end__ = nf;
  int *__3_status__ = (int *)(malloc(sizeof(int ) * (__local_block_3_end__ - __local_block_3_start__)));
  memset(__3_status__,0,sizeof(int ) * (__local_block_3_end__ - __local_block_3_start__));
  do {
    for (k = __local_block_0_start__; k < __local_block_0_end__; k++) 
      if (__0_status__[k - __local_block_0_start__] == 0) {
        __0_status__[k - __local_block_0_start__] = 1;
        add_vertex(0,k);
{
          if (is_known(1,k) && is_known(6,get_elem(1,k))) 
            for (i = 0; i < get_elem(6,get_elem(1,k)); i++) {
              if (is_known(1,k) && is_known(7,get_elem(1,k))) 
                for (j = 0; j < get_elem(7,get_elem(1,k)); j++) {
                  if (is_known(1,k) && is_known(9,get_elem(1,k)) && (is_known(1,k) && is_known(7,get_elem(1,k)))) 
                    add_pin_to_net(1,(get_elem(9,get_elem(1,k)) + (i * get_elem(7,get_elem(1,k)))) + j,1,0);
                  else 
                    __0_status__[k - __local_block_0_start__] = 0;;
                }
              else {
                __0_status__[k - __local_block_0_start__] = 0;;
              };
            }
          else {
            __0_status__[k - __local_block_0_start__] = 0;;
          }
          if (is_known(1,k) && is_known(10,get_elem(1,k)) && (is_known(1,k) && is_known(10,get_elem(1,k) + 1))) 
            for (p = get_elem(10,get_elem(1,k)); p < get_elem(10,get_elem(1,k) + 1); p++) {{
                if (is_known(16,p)) 
                  if (get_elem(16,p) == 1) {
                    if (is_known(11,p) && is_known(12,p)) 
                      for (i = get_elem(11,p); i < get_elem(12,p); i++) {
                        if (is_known(13,p) && is_known(14,p)) 
                          for (j = get_elem(13,p); j < get_elem(14,p); j++) {{
                              if (is_known(15,p) && is_known(9,get_elem(15,p)) && (is_known(15,p) && is_known(2,get_elem(15,p)) && (is_known(15,p) && is_known(7,get_elem(15,p)))) && (is_known(15,p) && is_known(4,get_elem(15,p)))) 
                                add_pin_to_net(1,((get_elem(9,get_elem(15,p)) + (((i * 2) - (get_elem(2,get_elem(15,p)) * 2)) * get_elem(7,get_elem(15,p)))) + (j * 2)) - (get_elem(4,get_elem(15,p)) * 2),0,0);
                              else 
                                __0_status__[k - __local_block_0_start__] = 0;
                              if (is_known(15,p) && is_known(9,get_elem(15,p)) && (is_known(15,p) && is_known(2,get_elem(15,p)) && (is_known(15,p) && is_known(7,get_elem(15,p)))) && (is_known(15,p) && is_known(4,get_elem(15,p)))) 
                                add_pin_to_net(1,((get_elem(9,get_elem(15,p)) + ((((i * 2) + 1) - (get_elem(2,get_elem(15,p)) * 2)) * get_elem(7,get_elem(15,p)))) + (j * 2)) - (get_elem(4,get_elem(15,p)) * 2),0,0);
                              else 
                                __0_status__[k - __local_block_0_start__] = 0;
                              if (is_known(15,p) && is_known(9,get_elem(15,p)) && (is_known(15,p) && is_known(2,get_elem(15,p)) && (is_known(15,p) && is_known(7,get_elem(15,p)))) && (is_known(15,p) && is_known(4,get_elem(15,p)))) 
                                add_pin_to_net(1,(((get_elem(9,get_elem(15,p)) + (((i * 2) - (get_elem(2,get_elem(15,p)) * 2)) * get_elem(7,get_elem(15,p)))) + (j * 2)) + 1) - (get_elem(4,get_elem(15,p)) * 2),0,0);
                              else 
                                __0_status__[k - __local_block_0_start__] = 0;
                              if (is_known(15,p) && is_known(9,get_elem(15,p)) && (is_known(15,p) && is_known(2,get_elem(15,p)) && (is_known(15,p) && is_known(7,get_elem(15,p)))) && (is_known(15,p) && is_known(4,get_elem(15,p)))) 
                                add_pin_to_net(1,(((get_elem(9,get_elem(15,p)) + ((((i * 2) + 1) - (get_elem(2,get_elem(15,p)) * 2)) * get_elem(7,get_elem(15,p)))) + (j * 2)) + 1) - (get_elem(4,get_elem(15,p)) * 2),0,0);
                              else 
                                __0_status__[k - __local_block_0_start__] = 0;;
			      if( is_known(1,k) && is_known(2,get_elem(1,k)) && is_known(1,k) && is_known(4,get_elem(1,k)) )
				;
			      else
				__0_status__[k-__local_block_0_start__] = 0;
			      if( is_known(1,k) && is_known(2,get_elem(1,k)) && is_known(1,k) && is_known(4,get_elem(1,k)) )
				;
			      else
				__0_status__[k-__local_block_0_start__] = 0;
			      if( is_known(1,k) && is_known(2,get_elem(1,k)) && is_known(1,k) && is_known(4,get_elem(1,k)) )
				;
			      else
				__0_status__[k-__local_block_0_start__] = 0;
			      if( is_known(1,k) && is_known(2,get_elem(1,k)) && is_known(1,k) && is_known(4,get_elem(1,k)) )
				;
			      else
				__0_status__[k-__local_block_0_start__] = 0;
                            };
                          }
                        else {
                          __0_status__[k - __local_block_0_start__] = 0;;
                        };
                      }
                    else {
                      __0_status__[k - __local_block_0_start__] = 0;;
                    };
                  }
                  else {
                    if (is_known(11,p) && is_known(12,p)) 
                      for (i = get_elem(11,p); i < get_elem(12,p); i++) {
                        if (is_known(13,p) && is_known(14,p)) 
                          for (j = get_elem(13,p); j < get_elem(14,p); j++) {{
                              if (is_known(15,p) && is_known(9,get_elem(15,p)) && (is_known(15,p) && is_known(2,get_elem(15,p)) && (is_known(15,p) && is_known(7,get_elem(15,p)))) && (is_known(15,p) && is_known(4,get_elem(15,p)))) 
                                add_pin_to_net(1,((get_elem(9,get_elem(15,p)) + ((i - get_elem(2,get_elem(15,p))) * get_elem(7,get_elem(15,p)))) + j) - get_elem(4,get_elem(15,p)),0,0);
                              else 
                                __0_status__[k - __local_block_0_start__] = 0;;
			      if( is_known(1,k) && is_known(2,get_elem(1,k)) && is_known(1,k) && is_known(4,get_elem(1,k)) )
				;
			      else
				__0_status__[k-__local_block_0_start__] = 0;
			      if( is_known(1,k) && is_known(2,get_elem(1,k)) && is_known(1,k) && is_known(4,get_elem(1,k)) )
				;
			      else
				__0_status__[k-__local_block_0_start__] = 0;
			      if( is_known(1,k) && is_known(2,get_elem(1,k)) && is_known(1,k) && is_known(4,get_elem(1,k)) )
				;
			      else
				__0_status__[k-__local_block_0_start__] = 0;
			      if( is_known(1,k) && is_known(2,get_elem(1,k)) && is_known(1,k) && is_known(4,get_elem(1,k)) )
				;
			      else
				__0_status__[k-__local_block_0_start__] = 0;

                            };
                          }
                        else {
                          __0_status__[k - __local_block_0_start__] = 0;;
                        };
                      }
                    else {
                      __0_status__[k - __local_block_0_start__] = 0;;
                    };
                  }
                else {
                  __0_status__[k - __local_block_0_start__] = 0;;
                };
              };
            }
          else {
            __0_status__[k - __local_block_0_start__] = 0;;
          }
          if (is_known(1,k) && is_known(6,get_elem(1,k))) 
            for (i = 0; i < get_elem(6,get_elem(1,k)); i++) {
              if (is_known(1,k) && is_known(7,get_elem(1,k))) 
                for (j = 0; j < get_elem(7,get_elem(1,k)); j++) {
                  if (is_known(1,k) && is_known(9,get_elem(1,k)) && (is_known(1,k) && is_known(7,get_elem(1,k)))) 
                    add_pin_to_net(0,(get_elem(9,get_elem(1,k)) + (i * get_elem(7,get_elem(1,k)))) + j,1,0);
                  else 
                    __0_status__[k - __local_block_0_start__] = 0;
                  if (is_known(1,k) && is_known(9,get_elem(1,k)) && (is_known(1,k) && is_known(7,get_elem(1,k)))) 
                    add_pin_to_net(2,(get_elem(9,get_elem(1,k)) + (i * get_elem(7,get_elem(1,k)))) + j,1,0);
                  else 
                    __0_status__[k - __local_block_0_start__] = 0;;
                }
              else {
                __0_status__[k - __local_block_0_start__] = 0;;
              };
            }
          else {
            __0_status__[k - __local_block_0_start__] = 0;;
          };
        };
      }
      else ;
    for (k = __local_block_1_start__; k < __local_block_1_end__; k++) 
      if (__1_status__[k - __local_block_1_start__] == 0) {
        __1_status__[k - __local_block_1_start__] = 1;
        add_vertex(0,k);
{
          if (is_known(1,k) && is_known(6,get_elem(1,k))) 
            for (i = 0; i < get_elem(6,get_elem(1,k)); i++) {
              if (is_known(1,k) && is_known(7,get_elem(1,k))) 
                for (j = 0; j < get_elem(7,get_elem(1,k)); j++) {
                  if (is_known(1,k) && is_known(9,get_elem(1,k)) && (is_known(1,k) && is_known(7,get_elem(1,k)))) 
                    add_pin_to_net(0,(get_elem(9,get_elem(1,k)) + (i * get_elem(7,get_elem(1,k)))) + j,1,0);
                  else 
                    __1_status__[k - __local_block_1_start__] = 0;;
                }
              else {
                __1_status__[k - __local_block_1_start__] = 0;;
              };
            }
          else {
            __1_status__[k - __local_block_1_start__] = 0;;
          }
          if (is_known(1,k) && is_known(10,get_elem(1,k)) && (is_known(1,k) && is_known(10,get_elem(1,k) + 1))) 
            for (p = get_elem(10,get_elem(1,k)); p < get_elem(10,get_elem(1,k) + 1); p++) {{
                if (is_known(16,p)) 
                  if (get_elem(16,p) == 1) {
                    if (is_known(11,p) && is_known(12,p)) 
                      for (i = get_elem(11,p); i < get_elem(12,p); i++) {
                        if (is_known(13,p) && is_known(14,p)) 
                          for (j = get_elem(13,p); j < get_elem(14,p); j++) {{
                              if (is_known(15,p) && is_known(9,get_elem(15,p)) && (is_known(15,p) && is_known(2,get_elem(15,p)) && (is_known(15,p) && is_known(7,get_elem(15,p)))) && (is_known(15,p) && is_known(4,get_elem(15,p)))) 
                                add_pin_to_net(0,((get_elem(9,get_elem(15,p)) + (((i * 2) - (get_elem(2,get_elem(15,p)) * 2)) * get_elem(7,get_elem(15,p)))) + (j * 2)) - (get_elem(4,get_elem(15,p)) * 2),0,0);
                              else 
                                __1_status__[k - __local_block_1_start__] = 0;
                              if (is_known(15,p) && is_known(9,get_elem(15,p)) && (is_known(15,p) && is_known(2,get_elem(15,p)) && (is_known(15,p) && is_known(7,get_elem(15,p)))) && (is_known(15,p) && is_known(4,get_elem(15,p)))) 
                                add_pin_to_net(0,((get_elem(9,get_elem(15,p)) + ((((i * 2) - (get_elem(2,get_elem(15,p)) * 2)) + 1) * get_elem(7,get_elem(15,p)))) + (j * 2)) - (get_elem(4,get_elem(15,p)) * 2),0,0);
                              else 
                                __1_status__[k - __local_block_1_start__] = 0;
                              if (is_known(15,p) && is_known(9,get_elem(15,p)) && (is_known(15,p) && is_known(2,get_elem(15,p)) && (is_known(15,p) && is_known(7,get_elem(15,p)))) && (is_known(15,p) && is_known(4,get_elem(15,p)))) 
                                add_pin_to_net(0,(((get_elem(9,get_elem(15,p)) + (((i * 2) - (get_elem(2,get_elem(15,p)) * 2)) * get_elem(7,get_elem(15,p)))) + (j * 2)) - (get_elem(4,get_elem(15,p)) * 2)) + 1,0,0);
                              else 
                                __1_status__[k - __local_block_1_start__] = 0;
                              if (is_known(15,p) && is_known(9,get_elem(15,p)) && (is_known(15,p) && is_known(2,get_elem(15,p)) && (is_known(15,p) && is_known(7,get_elem(15,p)))) && (is_known(15,p) && is_known(4,get_elem(15,p)))) 
                                add_pin_to_net(0,(((get_elem(9,get_elem(15,p)) + ((((i * 2) - (get_elem(2,get_elem(15,p)) * 2)) + 1) * get_elem(7,get_elem(15,p)))) + (j * 2)) - (get_elem(4,get_elem(15,p)) * 2)) + 1,0,0);
                              else 
                                __1_status__[k - __local_block_1_start__] = 0;;
			      if( is_known(1,k) && is_known(2,get_elem(1,k)) && is_known(1,k) && is_known(4,get_elem(1,k)) )
				;
			      else
				__1_status__[k-__local_block_1_start__] = 0;
			      if( is_known(1,k) && is_known(2,get_elem(1,k)) && is_known(1,k) && is_known(4,get_elem(1,k)) )
				;
			      else
				__1_status__[k-__local_block_1_start__] = 0;
			      if( is_known(1,k) && is_known(2,get_elem(1,k)) && is_known(1,k) && is_known(4,get_elem(1,k)) )
				;
			      else
				__1_status__[k-__local_block_1_start__] = 0;
			      if( is_known(1,k) && is_known(2,get_elem(1,k)) && is_known(1,k) && is_known(4,get_elem(1,k)) )
				;
			      else
				__1_status__[k-__local_block_1_start__] = 0;
                            };
                          }
                        else {
                          __1_status__[k - __local_block_1_start__] = 0;;
                        };
                      }
                    else {
                      __1_status__[k - __local_block_1_start__] = 0;;
                    };
                  }
                  else {
                    if (is_known(11,p) && is_known(12,p)) 
                      for (i = get_elem(11,p); i < get_elem(12,p); i++) {
                        if (is_known(13,p) && is_known(14,p)) 
                          for (j = get_elem(13,p); j < get_elem(14,p); j++) {{
                              if (is_known(15,p) && is_known(9,get_elem(15,p)) && (is_known(15,p) && is_known(2,get_elem(15,p)) && (is_known(15,p) && is_known(7,get_elem(15,p)))) && (is_known(15,p) && is_known(4,get_elem(15,p)))) 
                                add_pin_to_net(0,((get_elem(9,get_elem(15,p)) + ((i - get_elem(2,get_elem(15,p))) * get_elem(7,get_elem(15,p)))) + j) - get_elem(4,get_elem(15,p)),0,0);
                              else 
                                __1_status__[k - __local_block_1_start__] = 0;;
			      if( is_known(1,k) && is_known(2,get_elem(1,k)) && is_known(1,k) && is_known(4,get_elem(1,k)) )
				;
			      else
				__1_status__[k-__local_block_1_start__] = 0;
			      if( is_known(1,k) && is_known(2,get_elem(1,k)) && is_known(1,k) && is_known(4,get_elem(1,k)) )
				;
			      else
				__1_status__[k-__local_block_1_start__] = 0;
			      if( is_known(1,k) && is_known(2,get_elem(1,k)) && is_known(1,k) && is_known(4,get_elem(1,k)) )
				;
			      else
				__1_status__[k-__local_block_1_start__] = 0;
			      if( is_known(1,k) && is_known(2,get_elem(1,k)) && is_known(1,k) && is_known(4,get_elem(1,k)) )
				;
			      else
				__1_status__[k-__local_block_1_start__] = 0;
                            };
                          }
                        else {
                          __1_status__[k - __local_block_1_start__] = 0;;
                        };
                      }
                    else {
                      __1_status__[k - __local_block_1_start__] = 0;;
                    };
                  }
                else {
                  __1_status__[k - __local_block_1_start__] = 0;;
                };
              };
            }
          else {
            __1_status__[k - __local_block_1_start__] = 0;;
          }
          if (is_known(1,k) && is_known(6,get_elem(1,k))) 
            for (i = 0; i < get_elem(6,get_elem(1,k)); i++) {
              if (is_known(1,k) && is_known(7,get_elem(1,k))) 
                for (j = 0; j < get_elem(7,get_elem(1,k)); j++) {
                  if (is_known(1,k) && is_known(9,get_elem(1,k)) && (is_known(1,k) && is_known(7,get_elem(1,k)))) 
                    add_pin_to_net(2,(get_elem(9,get_elem(1,k)) + (i * get_elem(7,get_elem(1,k)))) + j,1,0);
                  else 
                    __1_status__[k - __local_block_1_start__] = 0;;
                }
              else {
                __1_status__[k - __local_block_1_start__] = 0;;
              };
            }
          else {
            __1_status__[k - __local_block_1_start__] = 0;;
          }
          if (is_known(1,k) && is_known(2,get_elem(1,k)) && (is_known(1,k) && is_known(3,get_elem(1,k)))) 
            for (i = get_elem(2,get_elem(1,k)); i < get_elem(3,get_elem(1,k)); i++) {
              if (is_known(1,k) && is_known(4,get_elem(1,k)) && (is_known(1,k) && is_known(5,get_elem(1,k)))) 
                for (j = get_elem(4,get_elem(1,k)); j < get_elem(5,get_elem(1,k)); j++) {{
		    if( is_known(1,k) && is_known(2,get_elem(1,k)) && is_known(1,k) && is_known(4,get_elem(1,k)) )
		      ;
		    else
		      __1_status__[k-__local_block_1_start__] = 0;
		    if( is_known(1,k) && is_known(2,get_elem(1,k)) && is_known(1,k) && is_known(4,get_elem(1,k)) )
		      ;
		    else
		      __1_status__[k-__local_block_1_start__] = 0;
		    if( is_known(1,k) && is_known(2,get_elem(1,k)) && is_known(1,k) && is_known(4,get_elem(1,k)) )
		      ;
		    else
		      __1_status__[k-__local_block_1_start__] = 0;
		    if( is_known(1,k) && is_known(2,get_elem(1,k)) && is_known(1,k) && is_known(4,get_elem(1,k)) )
		      ;
		    else
		      __1_status__[k-__local_block_1_start__] = 0;
                    if (is_known(8,k) && is_known(9,get_elem(8,k)) && (is_known(8,k) && is_known(2,get_elem(8,k)) && (is_known(8,k) && is_known(7,get_elem(8,k)))) && (is_known(8,k) && is_known(4,get_elem(8,k)))) 
                      add_pin_to_net(2,(get_elem(9,get_elem(8,k)) + ((i - get_elem(2,get_elem(8,k))) * get_elem(7,get_elem(8,k)))) + (j - get_elem(4,get_elem(8,k))),0,0);
                    else 
                      __1_status__[k - __local_block_1_start__] = 0;
                    if (is_known(8,k) && is_known(9,get_elem(8,k)) && (is_known(8,k) && is_known(2,get_elem(8,k)) && (is_known(8,k) && is_known(7,get_elem(8,k)))) && (is_known(8,k) && is_known(4,get_elem(8,k)))) 
                      add_pin_to_net(1,(get_elem(9,get_elem(8,k)) + ((i - get_elem(2,get_elem(8,k))) * get_elem(7,get_elem(8,k)))) + (j - get_elem(4,get_elem(8,k))),0,0);
                    else 
                      __1_status__[k - __local_block_1_start__] = 0;;
                  };
                }
              else {
                __1_status__[k - __local_block_1_start__] = 0;;
              };
            }
          else {
            __1_status__[k - __local_block_1_start__] = 0;;
          };
        };
      }
      else ;
    for (k = __local_block_2_start__; k < __local_block_2_end__; k++) 
      if (__2_status__[k - __local_block_2_start__] == 0) {
        __2_status__[k - __local_block_2_start__] = 1;
        add_vertex(1,k);
{
          if (is_known(0,k) && is_known(6,coarse_boxes[k])) 
            for (i = 0; i < get_elem(6,get_elem(0,k)); i++) {
              if (is_known(0,k) && is_known(7,coarse_boxes[k])) 
                for (j = 0; j < get_elem(7,get_elem(0,k)); j++) {
                  if (is_known(0,k) && is_known(9,coarse_boxes[k]) && (is_known(0,k) && is_known(7,coarse_boxes[k]))) 
                    add_pin_to_net(1,(get_elem(9,get_elem(0,k)) + (i * get_elem(7,get_elem(0,k)))) + j,1,0);
                  else 
                    __2_status__[k - __local_block_2_start__] = 0;;
                }
              else {
                __2_status__[k - __local_block_2_start__] = 0;;
              };
            }
          else {
            __2_status__[k - __local_block_2_start__] = 0;;
          }
          if (is_known(0,k) && is_known(10,coarse_boxes[k]) && (is_known(0,k) && is_known(10,coarse_boxes[k] + 1))) 
            for (p = get_elem(10,get_elem(0,k)); p < get_elem(10,get_elem(0,k) + 1); p++) {
              if (is_known(16,p)) 
                if (get_elem(16,p) == -1) {{
                    if (is_known(11,p) && is_known(12,p)) 
                      for (i = get_elem(11,p); i < get_elem(12,p); i++) {
                        if (is_known(13,p) && is_known(14,p)) 
                          for (j = get_elem(13,p); j < get_elem(14,p); j++) {
			    if( is_known(0,k) && is_known(2,get_elem(0,k)) && is_known(0,k) && is_known(4,get_elem(0,k)) )
			      ;
			    else
			      __2_status__[k-__local_block_2_start__] = 0;
			  }
                        else {
                          __2_status__[k - __local_block_2_start__] = 0;;
                        };
                      }
                    else {
                      __2_status__[k - __local_block_2_start__] = 0;;
                    };
                  };
                }
                else {{
                    if (is_known(11,p) && is_known(12,p)) 
                      for (i = get_elem(11,p); i < get_elem(12,p); i++) {
                        if (is_known(13,p) && is_known(14,p)) 
                          for (j = get_elem(13,p); j < get_elem(14,p); j++) {
                            if (is_known(15,p) && is_known(9,get_elem(15,p)) && (is_known(15,p) && is_known(2,get_elem(15,p)) && (is_known(15,p) && is_known(7,get_elem(15,p)))) && (is_known(15,p) && is_known(4,get_elem(15,p)))) 
                              add_pin_to_net(1,((get_elem(9,get_elem(15,p)) + ((i - get_elem(2,get_elem(15,p))) * get_elem(7,get_elem(15,p)))) + j) - get_elem(4,get_elem(15,p)),1,0);
                            else 
                              __2_status__[k - __local_block_2_start__] = 0;;
			    if( is_known(0,k) && is_known(2,get_elem(0,k)) && is_known(0,k) && is_known(4,get_elem(0,k)) )
			      ;
			    else
			      __2_status__[k-__local_block_2_start__] = 0;
                          }
                        else {
                          __2_status__[k - __local_block_2_start__] = 0;;
                        };
                      }
                    else {
                      __2_status__[k - __local_block_2_start__] = 0;;
                    };
                  };
                }
              else {
                __2_status__[k - __local_block_2_start__] = 0;;
              };
            }
          else {
            __2_status__[k - __local_block_2_start__] = 0;;
          }
          if (is_known(0,k) && is_known(6,coarse_boxes[k])) 
            for (i = 0; i < get_elem(6,get_elem(0,k)); i++) {
              if (is_known(0,k) && is_known(7,coarse_boxes[k])) 
                for (j = 0; j < get_elem(7,get_elem(0,k)); j++) {
                  if (is_known(0,k) && is_known(9,coarse_boxes[k]) && (is_known(0,k) && is_known(7,coarse_boxes[k]))) 
                    add_pin_to_net(0,(get_elem(9,get_elem(0,k)) + (i * get_elem(7,get_elem(0,k)))) + j,1,0);
                  else 
                    __2_status__[k - __local_block_2_start__] = 0;
                  if (is_known(0,k) && is_known(9,coarse_boxes[k]) && (is_known(0,k) && is_known(7,coarse_boxes[k]))) 
                    add_pin_to_net(2,(get_elem(9,get_elem(0,k)) + (i * get_elem(7,get_elem(0,k)))) + j,1,0);
                  else 
                    __2_status__[k - __local_block_2_start__] = 0;;
                }
              else {
                __2_status__[k - __local_block_2_start__] = 0;;
              };
            }
          else {
            __2_status__[k - __local_block_2_start__] = 0;;
          };
        };
      }
      else ;
    for (k = __local_block_3_start__; k < __local_block_3_end__; k++) 
      if (__3_status__[k - __local_block_3_start__] == 0) {
        __3_status__[k - __local_block_3_start__] = 1;
        add_vertex(0,k);
{
          if (is_known(1,k) && is_known(2,get_elem(1,k)) && (is_known(1,k) && is_known(3,get_elem(1,k)))) 
            for (i = get_elem(2,get_elem(1,k)); i < get_elem(3,get_elem(1,k)); i++) {
              if (is_known(1,k) && is_known(4,get_elem(1,k)) && (is_known(1,k) && is_known(5,get_elem(1,k)))) 
                for (j = get_elem(4,get_elem(1,k)); j < get_elem(5,get_elem(1,k)); j++) {{
                    if (is_known(1,k) && is_known(9,get_elem(1,k)) && (is_known(1,k) && is_known(2,get_elem(1,k)) && (is_known(1,k) && is_known(7,get_elem(1,k)))) && (is_known(1,k) && is_known(4,get_elem(1,k)))) 
                      add_pin_to_net(1,(get_elem(9,get_elem(1,k)) + (((i - get_elem(2,get_elem(1,k))) * 2) * get_elem(7,get_elem(1,k)))) + ((j - get_elem(4,get_elem(1,k))) * 2),0,0);
                    else 
                      __3_status__[k - __local_block_3_start__] = 0;
                    if (is_known(8,k) && is_known(9,get_elem(8,k)) && (is_known(8,k) && is_known(2,get_elem(8,k)) && (is_known(8,k) && is_known(7,get_elem(8,k)))) && (is_known(8,k) && is_known(4,get_elem(8,k)))) 
                      add_pin_to_net(0,((get_elem(9,get_elem(8,k)) + ((i - get_elem(2,get_elem(8,k))) * get_elem(7,get_elem(8,k)))) + j) - get_elem(4,get_elem(8,k)),0,0);
                    else 
                      __3_status__[k - __local_block_3_start__] = 0;
                    if (is_known(1,k) && is_known(9,get_elem(1,k)) && (is_known(1,k) && is_known(2,get_elem(1,k)) && (is_known(1,k) && is_known(7,get_elem(1,k)))) && (is_known(1,k) && is_known(4,get_elem(1,k)))) 
                      add_pin_to_net(1,(get_elem(9,get_elem(1,k)) + ((((i - get_elem(2,get_elem(1,k))) * 2) + 1) * get_elem(7,get_elem(1,k)))) + ((j - get_elem(4,get_elem(1,k))) * 2),0,0);
                    else 
                      __3_status__[k - __local_block_3_start__] = 0;
                    if (is_known(1,k) && is_known(9,get_elem(1,k)) && (is_known(1,k) && is_known(2,get_elem(1,k)) && (is_known(1,k) && is_known(7,get_elem(1,k)))) && (is_known(1,k) && is_known(4,get_elem(1,k)))) 
                      add_pin_to_net(1,((get_elem(9,get_elem(1,k)) + (((i - get_elem(2,get_elem(1,k))) * 2) * get_elem(7,get_elem(1,k)))) + ((j - get_elem(4,get_elem(1,k))) * 2)) + 1,0,0);
                    else 
                      __3_status__[k - __local_block_3_start__] = 0;
                    if (is_known(1,k) && is_known(9,get_elem(1,k)) && (is_known(1,k) && is_known(2,get_elem(1,k)) && (is_known(1,k) && is_known(7,get_elem(1,k)))) && (is_known(1,k) && is_known(4,get_elem(1,k)))) 
                      add_pin_to_net(1,((get_elem(9,get_elem(1,k)) + ((((i - get_elem(2,get_elem(1,k))) * 2) + 1) * get_elem(7,get_elem(1,k)))) + ((j - get_elem(4,get_elem(1,k))) * 2)) + 1,0,0);
                    else 
                      __3_status__[k - __local_block_3_start__] = 0;;
                  };
                }
              else {
                __3_status__[k - __local_block_3_start__] = 0;;
              };
            }
          else {
            __3_status__[k - __local_block_3_start__] = 0;;
          };
        };
      }
      else ;;
  }while (!done_graph_gen());
  partition_hypergraph(2);;
  free(__0_status__);
  __0_status__ = ((int *)(malloc(sizeof(int ) * get_proc_iter_size(0))));
  memset(__0_status__,0,sizeof(int ) * get_proc_iter_size(0));
  free(__1_status__);
  __1_status__ = ((int *)(malloc(sizeof(int ) * get_proc_iter_size(0))));
  memset(__1_status__,0,sizeof(int ) * get_proc_iter_size(0));
  free(__2_status__);
  __2_status__ = ((int *)(malloc(sizeof(int ) * get_proc_iter_size(1))));
  memset(__2_status__,0,sizeof(int ) * get_proc_iter_size(1));
  free(__3_status__);
  __3_status__ = ((int *)(malloc(sizeof(int ) * get_proc_iter_size(0))));
  memset(__3_status__,0,sizeof(int ) * get_proc_iter_size(0));
  do {
    int __0_status_counter__ = 0;
    for (k = 0; k < nf; k++) {
      if (get_vertex_home(0,k) / __nthreads__ == __proc_id__) {
        if (__0_status__[__0_status_counter__] == 0) {
          __0_status__[__0_status_counter__] = 1;
{
            if (is_known(1,k) && is_known(6,get_elem(1,k))) 
              for (i = 0; i < get_elem(6,get_elem(1,k)); i++) {
                if (is_known(1,k) && is_known(7,get_elem(1,k))) 
                  for (j = 0; j < get_elem(7,get_elem(1,k)); j++) {
                    if (is_known(1,k) && is_known(9,get_elem(1,k)) && (is_known(1,k) && is_known(7,get_elem(1,k)))) ;
                    else 
                      __0_status__[__0_status_counter__] = 0;;
                  }
                else {
                  __0_status__[__0_status_counter__] = 0;;
                };
              }
            else {
              __0_status__[__0_status_counter__] = 0;;
            }
            if (is_known(1,k) && is_known(10,get_elem(1,k)) && (is_known(1,k) && is_known(10,get_elem(1,k) + 1))) 
              for (p = get_elem(10,get_elem(1,k)); p < get_elem(10,get_elem(1,k) + 1); p++) {{
                  if (is_known(16,p)) 
                    if (get_elem(16,p) == 1) {
                      if (is_known(11,p) && is_known(12,p)) 
                        for (i = get_elem(11,p); i < get_elem(12,p); i++) {
                          if (is_known(13,p) && is_known(14,p)) 
                            for (j = get_elem(13,p); j < get_elem(14,p); j++) {{
                                if (is_known(15,p) && is_known(9,get_elem(15,p)) && (is_known(15,p) && is_known(2,get_elem(15,p)) && (is_known(15,p) && is_known(7,get_elem(15,p)))) && (is_known(15,p) && is_known(4,get_elem(15,p)))) ;
                                else 
                                  __0_status__[__0_status_counter__] = 0;
                                if (is_known(15,p) && is_known(9,get_elem(15,p)) && (is_known(15,p) && is_known(2,get_elem(15,p)) && (is_known(15,p) && is_known(7,get_elem(15,p)))) && (is_known(15,p) && is_known(4,get_elem(15,p)))) ;
                                else 
                                  __0_status__[__0_status_counter__] = 0;
                                if (is_known(15,p) && is_known(9,get_elem(15,p)) && (is_known(15,p) && is_known(2,get_elem(15,p)) && (is_known(15,p) && is_known(7,get_elem(15,p)))) && (is_known(15,p) && is_known(4,get_elem(15,p)))) ;
                                else 
                                  __0_status__[__0_status_counter__] = 0;
                                if (is_known(15,p) && is_known(9,get_elem(15,p)) && (is_known(15,p) && is_known(2,get_elem(15,p)) && (is_known(15,p) && is_known(7,get_elem(15,p)))) && (is_known(15,p) && is_known(4,get_elem(15,p)))) ;
                                else 
                                  __0_status__[__0_status_counter__] = 0;;
				if( is_known(1,k) && is_known(2,get_elem(1,k)) && is_known(1,k) && is_known(4,get_elem(1,k)))
				  ;
				else
				  __0_status__[__0_status_counter__] = 0;;
				if( is_known(1,k) && is_known(2,get_elem(1,k)) && is_known(1,k) && is_known(4,get_elem(1,k)))
				  ;
				else
				  __0_status__[__0_status_counter__] = 0;;
				if( is_known(1,k) && is_known(2,get_elem(1,k)) && is_known(1,k) && is_known(4,get_elem(1,k)))
				  ;
				else
				  __0_status__[__0_status_counter__] = 0;;
				if( is_known(1,k) && is_known(2,get_elem(1,k)) && is_known(1,k) && is_known(4,get_elem(1,k)))
				  ;
				else
				  __0_status__[__0_status_counter__] = 0;;
                              };
                            }
                          else {
                            __0_status__[__0_status_counter__] = 0;;
                          };
                        }
                      else {
                        __0_status__[__0_status_counter__] = 0;;
                      };
                    }
                    else {
                      if (is_known(11,p) && is_known(12,p)) 
                        for (i = get_elem(11,p); i < get_elem(12,p); i++) {
                          if (is_known(13,p) && is_known(14,p)) 
                            for (j = get_elem(13,p); j < get_elem(14,p); j++) {{
                                if (is_known(15,p) && is_known(9,get_elem(15,p)) && (is_known(15,p) && is_known(2,get_elem(15,p)) && (is_known(15,p) && is_known(7,get_elem(15,p)))) && (is_known(15,p) && is_known(4,get_elem(15,p)))) ;
                                else 
                                  __0_status__[__0_status_counter__] = 0;;
				if( is_known(1,k) && is_known(2,get_elem(1,k)) && is_known(1,k) && is_known(4,get_elem(1,k)))
				  ;
				else
				  __0_status__[__0_status_counter__] = 0;;
				if( is_known(1,k) && is_known(2,get_elem(1,k)) && is_known(1,k) && is_known(4,get_elem(1,k)))
				  ;
				else
				  __0_status__[__0_status_counter__] = 0;;
				if( is_known(1,k) && is_known(2,get_elem(1,k)) && is_known(1,k) && is_known(4,get_elem(1,k)))
				  ;
				else
				  __0_status__[__0_status_counter__] = 0;;
				if( is_known(1,k) && is_known(2,get_elem(1,k)) && is_known(1,k) && is_known(4,get_elem(1,k)))
				  ;
				else
				  __0_status__[__0_status_counter__] = 0;;

                              };
                            }
                          else {
                            __0_status__[__0_status_counter__] = 0;;
                          };
                        }
                      else {
                        __0_status__[__0_status_counter__] = 0;;
                      };
                    }
                  else {
                    __0_status__[__0_status_counter__] = 0;;
                  };
                };
              }
            else {
              __0_status__[__0_status_counter__] = 0;;
            }
            if (is_known(1,k) && is_known(6,get_elem(1,k))) 
              for (i = 0; i < get_elem(6,get_elem(1,k)); i++) {
                if (is_known(1,k) && is_known(7,get_elem(1,k))) 
                  for (j = 0; j < get_elem(7,get_elem(1,k)); j++) {
                    if (is_known(1,k) && is_known(9,get_elem(1,k)) && (is_known(1,k) && is_known(7,get_elem(1,k)))) ;
                    else 
                      __0_status__[__0_status_counter__] = 0;
                    if (is_known(1,k) && is_known(9,get_elem(1,k)) && (is_known(1,k) && is_known(7,get_elem(1,k)))) ;
                    else 
                      __0_status__[__0_status_counter__] = 0;;
                  }
                else {
                  __0_status__[__0_status_counter__] = 0;;
                };
              }
            else {
              __0_status__[__0_status_counter__] = 0;;
            };
          };
        }
        else ;
        ++__0_status_counter__;;
      }
      else ;;
    }
    int __1_status_counter__ = 0;
    for (k = 0; k < nf; k++) {
      if (get_vertex_home(0,k) / __nthreads__ == __proc_id__) {
        if (__1_status__[__1_status_counter__] == 0) {
          __1_status__[__1_status_counter__] = 1;
{
            if (is_known(1,k) && is_known(6,get_elem(1,k))) 
              for (i = 0; i < get_elem(6,get_elem(1,k)); i++) {
                if (is_known(1,k) && is_known(7,get_elem(1,k))) 
                  for (j = 0; j < get_elem(7,get_elem(1,k)); j++) {
                    if (is_known(1,k) && is_known(9,get_elem(1,k)) && (is_known(1,k) && is_known(7,get_elem(1,k)))) ;
                    else 
                      __1_status__[__1_status_counter__] = 0;;
                  }
                else {
                  __1_status__[__1_status_counter__] = 0;;
                };
              }
            else {
              __1_status__[__1_status_counter__] = 0;;
            }
            if (is_known(1,k) && is_known(10,get_elem(1,k)) && (is_known(1,k) && is_known(10,get_elem(1,k) + 1))) 
              for (p = get_elem(10,get_elem(1,k)); p < get_elem(10,get_elem(1,k) + 1); p++) {{
                  if (is_known(16,p)) 
                    if (get_elem(16,p) == 1) {
                      if (is_known(11,p) && is_known(12,p)) 
                        for (i = get_elem(11,p); i < get_elem(12,p); i++) {
                          if (is_known(13,p) && is_known(14,p)) 
                            for (j = get_elem(13,p); j < get_elem(14,p); j++) {{
                                if (is_known(15,p) && is_known(9,get_elem(15,p)) && (is_known(15,p) && is_known(2,get_elem(15,p)) && (is_known(15,p) && is_known(7,get_elem(15,p)))) && (is_known(15,p) && is_known(4,get_elem(15,p)))) ;
                                else 
                                  __1_status__[__1_status_counter__] = 0;
                                if (is_known(15,p) && is_known(9,get_elem(15,p)) && (is_known(15,p) && is_known(2,get_elem(15,p)) && (is_known(15,p) && is_known(7,get_elem(15,p)))) && (is_known(15,p) && is_known(4,get_elem(15,p)))) ;
                                else 
                                  __1_status__[__1_status_counter__] = 0;
                                if (is_known(15,p) && is_known(9,get_elem(15,p)) && (is_known(15,p) && is_known(2,get_elem(15,p)) && (is_known(15,p) && is_known(7,get_elem(15,p)))) && (is_known(15,p) && is_known(4,get_elem(15,p)))) ;
                                else 
                                  __1_status__[__1_status_counter__] = 0;
                                if (is_known(15,p) && is_known(9,get_elem(15,p)) && (is_known(15,p) && is_known(2,get_elem(15,p)) && (is_known(15,p) && is_known(7,get_elem(15,p)))) && (is_known(15,p) && is_known(4,get_elem(15,p)))) ;
                                else 
                                  __1_status__[__1_status_counter__] = 0;;
				if( is_known(1,k) && is_known(2,get_elem(1,k)) && is_known(1,k) && is_known(4,get_elem(1,k)))
				  ;
				else
				  __1_status__[__1_status_counter__] = 0;;
				if( is_known(1,k) && is_known(2,get_elem(1,k)) && is_known(1,k) && is_known(4,get_elem(1,k)))
				  ;
				else
				  __1_status__[__1_status_counter__] = 0;;
				if( is_known(1,k) && is_known(2,get_elem(1,k)) && is_known(1,k) && is_known(4,get_elem(1,k)))
				  ;
				else
				  __1_status__[__1_status_counter__] = 0;;
				if( is_known(1,k) && is_known(2,get_elem(1,k)) && is_known(1,k) && is_known(4,get_elem(1,k)))
				  ;
				else
				  __1_status__[__1_status_counter__] = 0;;
				if( is_known(1,k) && is_known(2,get_elem(1,k)) && is_known(1,k) && is_known(4,get_elem(1,k)))
				  ;
				else
				  __1_status__[__1_status_counter__] = 0;;
                              };
                            }
                          else {
                            __1_status__[__1_status_counter__] = 0;;
                          };
                        }
                      else {
                        __1_status__[__1_status_counter__] = 0;;
                      };
                    }
                    else {
                      if (is_known(11,p) && is_known(12,p)) 
                        for (i = get_elem(11,p); i < get_elem(12,p); i++) {
                          if (is_known(13,p) && is_known(14,p)) 
                            for (j = get_elem(13,p); j < get_elem(14,p); j++) {{
                                if (is_known(15,p) && is_known(9,get_elem(15,p)) && (is_known(15,p) && is_known(2,get_elem(15,p)) && (is_known(15,p) && is_known(7,get_elem(15,p)))) && (is_known(15,p) && is_known(4,get_elem(15,p)))) ;
                                else 
                                  __1_status__[__1_status_counter__] = 0;;
				if( is_known(1,k) && is_known(2,get_elem(1,k)) && is_known(1,k) && is_known(4,get_elem(1,k)))
				  ;
				else
				  __1_status__[__1_status_counter__] = 0;;
				if( is_known(1,k) && is_known(2,get_elem(1,k)) && is_known(1,k) && is_known(4,get_elem(1,k)))
				  ;
				else
				  __1_status__[__1_status_counter__] = 0;;
				if( is_known(1,k) && is_known(2,get_elem(1,k)) && is_known(1,k) && is_known(4,get_elem(1,k)))
				  ;
				else
				  __1_status__[__1_status_counter__] = 0;;
				if( is_known(1,k) && is_known(2,get_elem(1,k)) && is_known(1,k) && is_known(4,get_elem(1,k)))
				  ;
				else
				  __1_status__[__1_status_counter__] = 0;;

                              };
                            }
                          else {
                            __1_status__[__1_status_counter__] = 0;;
                          };
                        }
                      else {
                        __1_status__[__1_status_counter__] = 0;;
                      };
                    }
                  else {
                    __1_status__[__1_status_counter__] = 0;;
                  };
                };
              }
            else {
              __1_status__[__1_status_counter__] = 0;;
            }
            if (is_known(1,k) && is_known(6,get_elem(1,k))) 
              for (i = 0; i < get_elem(6,get_elem(1,k)); i++) {
                if (is_known(1,k) && is_known(7,get_elem(1,k))) 
                  for (j = 0; j < get_elem(7,get_elem(1,k)); j++) {
                    if (is_known(1,k) && is_known(9,get_elem(1,k)) && (is_known(1,k) && is_known(7,get_elem(1,k)))) ;
                    else 
                      __1_status__[__1_status_counter__] = 0;;
                  }
                else {
                  __1_status__[__1_status_counter__] = 0;;
                };
              }
            else {
              __1_status__[__1_status_counter__] = 0;;
            }
            if (is_known(1,k) && is_known(2,get_elem(1,k)) && (is_known(1,k) && is_known(3,get_elem(1,k)))) 
              for (i = get_elem(2,get_elem(1,k)); i < get_elem(3,get_elem(1,k)); i++) {
                if (is_known(1,k) && is_known(4,get_elem(1,k)) && (is_known(1,k) && is_known(5,get_elem(1,k)))) 
                  for (j = get_elem(4,get_elem(1,k)); j < get_elem(5,get_elem(1,k)); j++) {{
                      if (is_known(8,k) && is_known(9,get_elem(8,k)) && (is_known(8,k) && is_known(2,get_elem(8,k)) && (is_known(8,k) && is_known(7,get_elem(8,k)))) && (is_known(8,k) && is_known(4,get_elem(8,k)))) ;
                      else 
                        __1_status__[__1_status_counter__] = 0;
                      if (is_known(8,k) && is_known(9,get_elem(8,k)) && (is_known(8,k) && is_known(2,get_elem(8,k)) && (is_known(8,k) && is_known(7,get_elem(8,k)))) && (is_known(8,k) && is_known(4,get_elem(8,k)))) ;
                      else 
                        __1_status__[__1_status_counter__] = 0;;
		      if( is_known(1,k) && is_known(2,get_elem(1,k)) && is_known(1,k) && is_known(4,get_elem(1,k)))
			;
		      else
			__1_status__[__1_status_counter__] = 0;;

		      if( is_known(1,k) && is_known(2,get_elem(1,k)) && is_known(1,k) && is_known(4,get_elem(1,k)))
			;
		      else
			__1_status__[__1_status_counter__] = 0;;
		      if( is_known(1,k) && is_known(2,get_elem(1,k)) && is_known(1,k) && is_known(4,get_elem(1,k)))
			;
		      else
			__1_status__[__1_status_counter__] = 0;;
		      if( is_known(1,k) && is_known(2,get_elem(1,k)) && is_known(1,k) && is_known(4,get_elem(1,k)))
			;
		      else
			__1_status__[__1_status_counter__] = 0;;
                    };
                  }
                else {
                  __1_status__[__1_status_counter__] = 0;;
                };
              }
            else {
              __1_status__[__1_status_counter__] = 0;;
            };
          };
        }
        else ;
        ++__1_status_counter__;;
      }
      else ;;
    }
    int __2_status_counter__ = 0;
    for (k = 0; k < nc; k++) {
      if (get_vertex_home(1,k) / __nthreads__ == __proc_id__) {
        if (__2_status__[__2_status_counter__] == 0) {
          __2_status__[__2_status_counter__] = 1;
{
            if (is_known(0,k) && is_known(6,coarse_boxes[k])) 
              for (i = 0; i < get_elem(6,get_elem(0,k)); i++) {
                if (is_known(0,k) && is_known(7,coarse_boxes[k])) 
                  for (j = 0; j < get_elem(7,get_elem(0,k)); j++) {
                    if (is_known(0,k) && is_known(9,coarse_boxes[k]) && (is_known(0,k) && is_known(7,coarse_boxes[k]))) ;
                    else 
                      __2_status__[__2_status_counter__] = 0;;
                  }
                else {
                  __2_status__[__2_status_counter__] = 0;;
                };
              }
            else {
              __2_status__[__2_status_counter__] = 0;;
            }
            if (is_known(0,k) && is_known(10,coarse_boxes[k]) && (is_known(0,k) && is_known(10,coarse_boxes[k] + 1))) 
              for (p = get_elem(10,get_elem(0,k)); p < get_elem(10,get_elem(0,k) + 1); p++) {
                if (is_known(16,p)) 
                  if (get_elem(16,p) == -1) {{
                      if (is_known(11,p) && is_known(12,p)) 
                        for (i = get_elem(11,p); i < get_elem(12,p); i++) {
                          if (is_known(13,p) && is_known(14,p)) 
                            for (j = get_elem(13,p); j < get_elem(14,p); j++) {
			      if( is_known(0,k) && is_known(2,get_elem(0,k)) && is_known(0,k) && is_known(4,get_elem(0,k)))
				;
			      else
				__2_status__[__2_status_counter__] = 0;;

			    }
                          else {
                            __2_status__[__2_status_counter__] = 0;;
                          };
                        }
                      else {
                        __2_status__[__2_status_counter__] = 0;;
                      };
                    };
                  }
                  else {{
                      if (is_known(11,p) && is_known(12,p)) 
                        for (i = get_elem(11,p); i < get_elem(12,p); i++) {
                          if (is_known(13,p) && is_known(14,p)) 
                            for (j = get_elem(13,p); j < get_elem(14,p); j++) {
                              if (is_known(15,p) && is_known(9,get_elem(15,p)) && (is_known(15,p) && is_known(2,get_elem(15,p)) && (is_known(15,p) && is_known(7,get_elem(15,p)))) && (is_known(15,p) && is_known(4,get_elem(15,p)))) ;
                              else 
                                __2_status__[__2_status_counter__] = 0;;
			      if( is_known(0,k) && is_known(2,get_elem(0,k)) && is_known(0,k) && is_known(4,get_elem(0,k)))
				;
			      else
				__2_status__[__2_status_counter__] = 0;;
                            }
                          else {
                            __2_status__[__2_status_counter__] = 0;;
                          };
                        }
                      else {
                        __2_status__[__2_status_counter__] = 0;;
                      };
                    };
                  }
                else {
                  __2_status__[__2_status_counter__] = 0;;
                };
              }
            else {
              __2_status__[__2_status_counter__] = 0;;
            }
            if (is_known(0,k) && is_known(6,coarse_boxes[k])) 
              for (i = 0; i < get_elem(6,get_elem(0,k)); i++) {
                if (is_known(0,k) && is_known(7,coarse_boxes[k])) 
                  for (j = 0; j < get_elem(7,get_elem(0,k)); j++) {
                    if (is_known(0,k) && is_known(9,coarse_boxes[k]) && (is_known(0,k) && is_known(7,coarse_boxes[k]))) ;
                    else 
                      __2_status__[__2_status_counter__] = 0;
                    if (is_known(0,k) && is_known(9,coarse_boxes[k]) && (is_known(0,k) && is_known(7,coarse_boxes[k]))) ;
                    else 
                      __2_status__[__2_status_counter__] = 0;;
                  }
                else {
                  __2_status__[__2_status_counter__] = 0;;
                };
              }
            else {
              __2_status__[__2_status_counter__] = 0;;
            };
          };
        }
        else ;
        ++__2_status_counter__;;
      }
      else ;;
    }
    int __3_status_counter__ = 0;
    for (k = 0; k < nf; k++) {
      if (get_vertex_home(0,k) / __nthreads__ == __proc_id__) {
        if (__3_status__[__3_status_counter__] == 0) {
          __3_status__[__3_status_counter__] = 1;
{
            if (is_known(1,k) && is_known(2,get_elem(1,k)) && (is_known(1,k) && is_known(3,get_elem(1,k)))) 
              for (i = get_elem(2,get_elem(1,k)); i < get_elem(3,get_elem(1,k)); i++) {
                if (is_known(1,k) && is_known(4,get_elem(1,k)) && (is_known(1,k) && is_known(5,get_elem(1,k)))) 
                  for (j = get_elem(4,get_elem(1,k)); j < get_elem(5,get_elem(1,k)); j++) {{
                      if (is_known(1,k) && is_known(9,get_elem(1,k)) && (is_known(1,k) && is_known(2,get_elem(1,k)) && (is_known(1,k) && is_known(7,get_elem(1,k)))) && (is_known(1,k) && is_known(4,get_elem(1,k)))) ;
                      else 
                        __3_status__[__3_status_counter__] = 0;
                      if (is_known(8,k) && is_known(9,get_elem(8,k)) && (is_known(8,k) && is_known(2,get_elem(8,k)) && (is_known(8,k) && is_known(7,get_elem(8,k)))) && (is_known(8,k) && is_known(4,get_elem(8,k)))) ;
                      else 
                        __3_status__[__3_status_counter__] = 0;
                      if (is_known(1,k) && is_known(9,get_elem(1,k)) && (is_known(1,k) && is_known(2,get_elem(1,k)) && (is_known(1,k) && is_known(7,get_elem(1,k)))) && (is_known(1,k) && is_known(4,get_elem(1,k)))) ;
                      else 
                        __3_status__[__3_status_counter__] = 0;
                      if (is_known(1,k) && is_known(9,get_elem(1,k)) && (is_known(1,k) && is_known(2,get_elem(1,k)) && (is_known(1,k) && is_known(7,get_elem(1,k)))) && (is_known(1,k) && is_known(4,get_elem(1,k)))) ;
                      else 
                        __3_status__[__3_status_counter__] = 0;
                      if (is_known(1,k) && is_known(9,get_elem(1,k)) && (is_known(1,k) && is_known(2,get_elem(1,k)) && (is_known(1,k) && is_known(7,get_elem(1,k)))) && (is_known(1,k) && is_known(4,get_elem(1,k)))) ;
                      else 
                        __3_status__[__3_status_counter__] = 0;;
                    };
                  }
                else {
                  __3_status__[__3_status_counter__] = 0;;
                };
              }
            else {
              __3_status__[__3_status_counter__] = 0;;
            };
          };
        }
        else ;
        ++__3_status_counter__;;
      }
      else ;;
    };
  }while (!done_graph_gen());
  free(__0_status__);
  free(__1_status__);
  free(__2_status__);
  free(__3_status__);;
  
#pragma omp parallel firstprivate (i,j,k,p,mimic_temp,mimic_temp2,mimic_temp3,read_temp,__thread_id__,__myid__,t)
{
  double *  phi_f_t = (double*) malloc(sizeof(double)*(max_f_y + 4)*(max_f_x + 4));
  double * residual = (double*) malloc(sizeof(double)*(max_f_y)*(max_f_x));
  double * phi_c_t = (double*) malloc(sizeof(double)*(max_c_y+2)*(max_c_x+2));

    __thread_id__ = omp_get_thread_num();
    __myid__ = __proc_id__ * __nthreads__ + __thread_id__;
    int __body_0__ = 0;
    int __loop_i_0__ = 0;
    int __body_i_0__ = 0;
    int __loop_i_1__ = 0;
    int __body_i_1__ = 0;
    int __loop_i_2__ = 0;
    int __body_i_2__ = 0;
    int __loop_i_3__ = 0;
    int __body_i_3__ = 0;
    int __loop_i_4__ = 0;
    int __body_i_4__ = 0;
    int __loop_i_5__ = 0;
    int __body_i_5__ = 0;
    int __loop_i_6__ = 0;
    int __body_i_6__ = 0;
    int __loop_i_7__ = 0;
    int __body_i_7__ = 0;
    int __loop_i_8__ = 0;
    int __body_i_8__ = 0;
    int __if_0__ = 0;
    int __then_0__ = 0;
    int __else_1__ = 0;
    int __body_1__ = 0;
    int __loop_i_9__ = 0;
    int __body_i_9__ = 0;
    int __loop_i_10__ = 0;
    int __body_i_10__ = 0;
    int __loop_i_11__ = 0;
    int __body_i_11__ = 0;
    int __loop_i_12__ = 0;
    int __body_i_12__ = 0;
    int __loop_i_13__ = 0;
    int __body_i_13__ = 0;
    int __loop_i_14__ = 0;
    int __body_i_14__ = 0;
    int __loop_i_15__ = 0;
    int __body_i_15__ = 0;
    int __loop_i_16__ = 0;
    int __body_i_16__ = 0;
    int __loop_i_17__ = 0;
    int __body_i_17__ = 0;
    int __loop_i_18__ = 0;
    int __body_i_18__ = 0;
    int __loop_i_19__ = 0;
    int __body_i_19__ = 0;
    int __if_2__ = 0;
    int __then_2__ = 0;
    int __else_3__ = 0;
    int __body_2__ = 0;
    int __loop_i_20__ = 0;
    int __body_i_20__ = 0;
    int __loop_i_21__ = 0;
    int __body_i_21__ = 0;
    int __loop_i_22__ = 0;
    int __body_i_22__ = 0;
    int __loop_i_23__ = 0;
    int __body_i_23__ = 0;
    int __loop_i_24__ = 0;
    int __body_i_24__ = 0;
    int __loop_i_25__ = 0;
    int __body_i_25__ = 0;
    int __loop_i_26__ = 0;
    int __body_i_26__ = 0;
    int __loop_i_27__ = 0;
    int __body_i_27__ = 0;
    int __loop_i_28__ = 0;
    int __body_i_28__ = 0;
    int __if_4__ = 0;
    int __then_4__ = 0;
    int __else_5__ = 0;
    int __body_3__ = 0;
    int __loop_i_29__ = 0;
    int __body_i_29__ = 0;
    int __loop_i_30__ = 0;
    int __body_i_30__ = 0;
    for (k = 0; k < nf; k++) {
      if (get_vertex_home(0,k) == __myid__) {{
          for (i = 0; i < get_elem(6,get_elem(1,k)); i++) {
            for (j = 0; j < get_elem(7,get_elem(1,k)); j++) {
              __body_i_1__ = __body_i_1__ + 1;;
            }
            __loop_i_1__ = __loop_i_1__ + 1;
            __body_i_0__ = __body_i_0__ + 1;;
          }
          __loop_i_0__ = __loop_i_0__ + 1;
          for (p = get_elem(10,get_elem(1,k)); p < get_elem(10,get_elem(1,k) + 1); p++) {{
              if (get_elem(16,p) == 1) {
                for (i = get_elem(11,p); i < get_elem(12,p); i++) {
                  for (j = get_elem(13,p); j < get_elem(14,p); j++) {{;
                    }
                    __body_i_4__ = __body_i_4__ + 1;;
                  }
                  __loop_i_4__ = __loop_i_4__ + 1;
                  __body_i_3__ = __body_i_3__ + 1;;
                }
                __loop_i_3__ = __loop_i_3__ + 1;
                __then_0__ = __then_0__ + 1;;
              }
              else {
                for (i = get_elem(11,p); i < get_elem(12,p); i++) {
                  for (j = get_elem(13,p); j < get_elem(14,p); j++) {{;
                    }
                    __body_i_6__ = __body_i_6__ + 1;;
                  }
                  __loop_i_6__ = __loop_i_6__ + 1;
                  __body_i_5__ = __body_i_5__ + 1;;
                }
                __loop_i_5__ = __loop_i_5__ + 1;
                __else_1__ = __else_1__ + 1;;
              }
              __if_0__ = __if_0__ + 1;;
            }
            __body_i_2__ = __body_i_2__ + 1;;
          }
          __loop_i_2__ = __loop_i_2__ + 1;
          for (i = 0; i < get_elem(6,get_elem(1,k)); i++) {
            for (j = 0; j < get_elem(7,get_elem(1,k)); j++) {
              __body_i_8__ = __body_i_8__ + 1;;
            }
            __loop_i_8__ = __loop_i_8__ + 1;
            __body_i_7__ = __body_i_7__ + 1;;
          }
          __loop_i_7__ = __loop_i_7__ + 1;;
        }
        __body_0__ = __body_0__ + 1;;
      }
      else ;;
    }
    for (k = 0; k < nf; k++) {
      if (get_vertex_home(0,k) == __myid__) {{
          for (i = 0; i < get_elem(6,get_elem(1,k)); i++) {
            for (j = 0; j < get_elem(7,get_elem(1,k)); j++) {
              __body_i_10__ = __body_i_10__ + 1;;
            }
            __loop_i_10__ = __loop_i_10__ + 1;
            __body_i_9__ = __body_i_9__ + 1;;
          }
          __loop_i_9__ = __loop_i_9__ + 1;
          for (p = get_elem(10,get_elem(1,k)); p < get_elem(10,get_elem(1,k) + 1); p++) {{
              if (get_elem(16,p) == 1) {
                for (i = get_elem(11,p); i < get_elem(12,p); i++) {
                  for (j = get_elem(13,p); j < get_elem(14,p); j++) {{;
                    }
                    __body_i_13__ = __body_i_13__ + 1;;
                  }
                  __loop_i_13__ = __loop_i_13__ + 1;
                  __body_i_12__ = __body_i_12__ + 1;;
                }
                __loop_i_12__ = __loop_i_12__ + 1;
                __then_2__ = __then_2__ + 1;;
              }
              else {
                for (i = get_elem(11,p); i < get_elem(12,p); i++) {
                  for (j = get_elem(13,p); j < get_elem(14,p); j++) {{;
                    }
                    __body_i_15__ = __body_i_15__ + 1;;
                  }
                  __loop_i_15__ = __loop_i_15__ + 1;
                  __body_i_14__ = __body_i_14__ + 1;;
                }
                __loop_i_14__ = __loop_i_14__ + 1;
                __else_3__ = __else_3__ + 1;;
              }
              __if_2__ = __if_2__ + 1;;
            }
            __body_i_11__ = __body_i_11__ + 1;;
          }
          __loop_i_11__ = __loop_i_11__ + 1;
          for (i = 0; i < get_elem(6,get_elem(1,k)); i++) {
            for (j = 0; j < get_elem(7,get_elem(1,k)); j++) {
              __body_i_17__ = __body_i_17__ + 1;;
            }
            __loop_i_17__ = __loop_i_17__ + 1;
            __body_i_16__ = __body_i_16__ + 1;;
          }
          __loop_i_16__ = __loop_i_16__ + 1;
          for (i = get_elem(2,get_elem(1,k)); i < get_elem(3,get_elem(1,k)); i++) {
            for (j = get_elem(4,get_elem(1,k)); j < get_elem(5,get_elem(1,k)); j++) {{;
              }
              __body_i_19__ = __body_i_19__ + 1;;
            }
            __loop_i_19__ = __loop_i_19__ + 1;
            __body_i_18__ = __body_i_18__ + 1;;
          }
          __loop_i_18__ = __loop_i_18__ + 1;;
        }
        __body_1__ = __body_1__ + 1;;
      }
      else ;;
    }
    for (k = 0; k < nc; k++) {
      if (get_vertex_home(1,k) == __myid__) {{
          for (i = 0; i < get_elem(6,get_elem(0,k)); i++) {
            for (j = 0; j < get_elem(7,get_elem(0,k)); j++) {
              __body_i_21__ = __body_i_21__ + 1;;
            }
            __loop_i_21__ = __loop_i_21__ + 1;
            __body_i_20__ = __body_i_20__ + 1;;
          }
          __loop_i_20__ = __loop_i_20__ + 1;
          for (p = get_elem(10,get_elem(0,k)); p < get_elem(10,get_elem(0,k) + 1); p++) {
            if (get_elem(16,p) == -1) {{
                for (i = get_elem(11,p); i < get_elem(12,p); i++) {
                  for (j = get_elem(13,p); j < get_elem(14,p); j++) {
                    __body_i_24__ = __body_i_24__ + 1;;
                  }
                  __loop_i_24__ = __loop_i_24__ + 1;
                  __body_i_23__ = __body_i_23__ + 1;;
                }
                __loop_i_23__ = __loop_i_23__ + 1;;
              }
              __then_4__ = __then_4__ + 1;;
            }
            else {{
                for (i = get_elem(11,p); i < get_elem(12,p); i++) {
                  for (j = get_elem(13,p); j < get_elem(14,p); j++) {
                    __body_i_26__ = __body_i_26__ + 1;;
                  }
                  __loop_i_26__ = __loop_i_26__ + 1;
                  __body_i_25__ = __body_i_25__ + 1;;
                }
                __loop_i_25__ = __loop_i_25__ + 1;;
              }
              __else_5__ = __else_5__ + 1;;
            }
            __if_4__ = __if_4__ + 1;
            __body_i_22__ = __body_i_22__ + 1;;
          }
          __loop_i_22__ = __loop_i_22__ + 1;
          for (i = 0; i < get_elem(6,get_elem(0,k)); i++) {
            for (j = 0; j < get_elem(7,get_elem(0,k)); j++) {
              __body_i_28__ = __body_i_28__ + 1;;
            }
            __loop_i_28__ = __loop_i_28__ + 1;
            __body_i_27__ = __body_i_27__ + 1;;
          }
          __loop_i_27__ = __loop_i_27__ + 1;;
        }
        __body_2__ = __body_2__ + 1;;
      }
      else ;;
    }
    for (k = 0; k < nf; k++) {
      if (get_vertex_home(0,k) == __myid__) {{
          for (i = get_elem(2,get_elem(1,k)); i < get_elem(3,get_elem(1,k)); i++) {
            for (j = get_elem(4,get_elem(1,k)); j < get_elem(5,get_elem(1,k)); j++) {{;
              }
              __body_i_30__ = __body_i_30__ + 1;;
            }
            __loop_i_30__ = __loop_i_30__ + 1;
            __body_i_29__ = __body_i_29__ + 1;;
          }
          __loop_i_29__ = __loop_i_29__ + 1;;
        }
        __body_3__ = __body_3__ + 1;;
      }
      else ;;
    };

    int * __trace_phi_f_t_0__ = (int*) malloc(__loop_i_1__ * sizeof(int));
    int __trace_phi_f_t_0__offset;

    int * __trace_phi_f_t_1__ = (int*) malloc(__body_i_4__ * sizeof(int));
    int * __trace_phi_f_t_2__ = (int*) malloc(__body_i_4__ * sizeof(int));
    int * __trace_phi_f_t_3__ = (int*) malloc(__body_i_4__ * sizeof(int));
    int * __trace_phi_f_t_4__ = (int*) malloc(__body_i_4__ * sizeof(int));

    int * __trace_phi_f_t_5__ = (int*) malloc(__body_i_6__ * sizeof(int));
    int * __trace_phi_f_t_6__ = (int*) malloc(__body_i_6__ * sizeof(int));
    int * __trace_phi_f_t_7__ = (int*) malloc(__body_i_6__ * sizeof(int));
    int * __trace_phi_f_t_8__ = (int*) malloc(__body_i_6__ * sizeof(int));

    int * __trace_phi_f_t_9__ = (int*) malloc(__loop_i_8__ * sizeof(int));
    int __trace_phi_f_t_9__offset;
    int * __trace_phi_f_t_10__ = (int*) malloc(__loop_i_8__ * sizeof(int));
    int __trace_phi_f_t_10__offset;
    int * __trace_phi_f_t_11__ = (int*) malloc(__loop_i_8__ * sizeof(int));
    int __trace_phi_f_t_11__offset;
    int * __trace_phi_f_t_12__ = (int*) malloc(__loop_i_8__ * sizeof(int));
    int __trace_phi_f_t_12__offset;
    int * __trace_phi_f_t_13__ = (int*) malloc(__loop_i_8__ * sizeof(int));
    int __trace_phi_f_t_13__offset;

    int * __trace_phi_f_t_14__ = (int*)malloc(__loop_i_10__ * sizeof(int));
    int __trace_phi_f_t_14__offset;
    
    int * __trace_phi_f_t_15__ = (int*) malloc(__body_i_13__ * sizeof(int));
    int * __trace_phi_f_t_16__ = (int*) malloc(__body_i_13__ * sizeof(int));
    int * __trace_phi_f_t_17__ = (int*) malloc(__body_i_13__ * sizeof(int));
    int * __trace_phi_f_t_18__ = (int*) malloc(__body_i_13__ * sizeof(int));

    int * __trace_phi_f_t_19__ = (int*) malloc(__body_i_15__ * sizeof(int));
    int * __trace_phi_f_t_20__ = (int*) malloc(__body_i_15__ * sizeof(int));
    int * __trace_phi_f_t_21__ = (int*) malloc(__body_i_15__ * sizeof(int));
    int * __trace_phi_f_t_22__ = (int*) malloc(__body_i_15__ * sizeof(int));

    int * __trace_phi_f_t_23__ = (int*) malloc(__loop_i_17__ * sizeof(int));
    int __trace_phi_f_t_23__offset;
    int * __trace_phi_f_t_24__ = (int*) malloc(__loop_i_17__ * sizeof(int));
    int __trace_phi_f_t_24__offset;
    int * __trace_phi_f_t_25__ = (int*) malloc(__loop_i_17__ * sizeof(int));
    int __trace_phi_f_t_25__offset;
    int * __trace_phi_f_t_26__ = (int*) malloc(__loop_i_17__ * sizeof(int));
    int __trace_phi_f_t_26__offset;
    int * __trace_phi_f_t_27__ = (int*) malloc(__loop_i_17__ * sizeof(int));
    int __trace_phi_f_t_27__offset;

    int *__trace_residual_0__ =  (int*) malloc(__loop_i_17__ * sizeof(int));
    int __trace_residual_0__offset;

    int * __trace_residual_1__ = (int*)malloc(__body_i_19__ * sizeof(int));
    int * __trace_residual_2__ = (int*)malloc(__body_i_19__ * sizeof(int));
    int * __trace_residual_3__ = (int*)malloc(__body_i_19__ * sizeof(int));
    int * __trace_residual_4__ = (int*)malloc(__body_i_19__ * sizeof(int));

    int * __trace_phi_c_t_0__ = (int*) malloc(__loop_i_21__ * sizeof(int));
    int __trace_phi_c_t_0__offset;
    int * __trace_phi_c_t_1__= (int*) malloc(__loop_i_24__ * sizeof(int));
    int __trace_phi_c_t_1__offset;
    int * __trace_phi_c_t_2__= (int*) malloc(__loop_i_26__ * sizeof(int));
    int __trace_phi_c_t_2__offset;
    
    int * __trace_phi_c_t_3__= (int*) malloc(__loop_i_28__ * sizeof(int));
    int __trace_phi_c_t_3__offset;
    int * __trace_phi_c_t_4__= (int*) malloc(__loop_i_28__ * sizeof(int));
    int __trace_phi_c_t_4__offset;
    int * __trace_phi_c_t_5__= (int*) malloc(__loop_i_28__ * sizeof(int));
    int __trace_phi_c_t_5__offset;
    int * __trace_phi_c_t_6__= (int*) malloc(__loop_i_28__ * sizeof(int));
    int __trace_phi_c_t_6__offset;
    int * __trace_phi_c_t_7__= (int*) malloc(__loop_i_28__ * sizeof(int));
    int __trace_phi_c_t_7__offset;

    int *__o_phi_j_6_ = (int *)(malloc(__loop_i_8__ * sizeof(int )));
    int __o_phi_j_6__offset;
    int *__o_phi_j_8_ = (int *)(malloc(__loop_i_10__ * sizeof(int )));
    int __o_phi_j_8__offset;
    int *__i_phi_j_9_ = (int *)(malloc(__body_i_13__ * sizeof(int )));
    int *__i_phi_j_10_ = (int *)(malloc(__body_i_13__ * sizeof(int )));
    int *__i_phi_j_11_ = (int *)(malloc(__body_i_13__ * sizeof(int )));
    int *__i_phi_j_12_ = (int *)(malloc(__body_i_13__ * sizeof(int )));
    int *__i_phi_j_13_ = (int *)(malloc(__body_i_15__ * sizeof(int )));
    int *__o_phi_j_19_ = (int *)(malloc(__loop_i_28__ * sizeof(int )));
    int __o_phi_j_19__offset;
    int *__i_phi_j_22_ = (int *)(malloc(__body_i_30__ * sizeof(int )));
    int *__o_phi_old_j_0_ = (int *)(malloc(__loop_i_1__ * sizeof(int )));
    int __o_phi_old_j_0__offset;
    int *__i_phi_old_j_1_ = (int *)(malloc(__body_i_4__ * sizeof(int )));
    int *__i_phi_old_j_2_ = (int *)(malloc(__body_i_4__ * sizeof(int )));
    int *__i_phi_old_j_3_ = (int *)(malloc(__body_i_4__ * sizeof(int )));
    int *__i_phi_old_j_4_ = (int *)(malloc(__body_i_4__ * sizeof(int )));
    int *__i_phi_old_j_5_ = (int *)(malloc(__body_i_6__ * sizeof(int )));
    int *__i_phi_old_j_16_ = (int *)(malloc(__body_i_19__ * sizeof(int )));
    int *__o_phi_old_j_17_ = (int *)(malloc(__loop_i_21__ * sizeof(int )));
    int __o_phi_old_j_17__offset;
    int *__o_phi_old_j_18_ = (int *)(malloc(__loop_i_26__ * sizeof(int )));
    int __o_phi_old_j_18__offset;
    int *__i_phi_old_j_21_ = (int *)(malloc(__body_i_30__ * sizeof(int )));
    int *__i_phi_old_j_23_ = (int *)(malloc(__body_i_30__ * sizeof(int )));
    int *__i_phi_old_j_24_ = (int *)(malloc(__body_i_30__ * sizeof(int )));
    int *__i_phi_old_j_25_ = (int *)(malloc(__body_i_30__ * sizeof(int )));
    int *__o_rho_j_7_ = (int *)(malloc(__loop_i_8__ * sizeof(int )));
    int __o_rho_j_7__offset;
    int *__o_rho_j_14_ = (int *)(malloc(__loop_i_17__ * sizeof(int )));
    int __o_rho_j_14__offset;
    int *__o_rho_j_20_ = (int *)(malloc(__loop_i_28__ * sizeof(int )));
    int __o_rho_j_20__offset;
    int *__i_rho_write_j_15_ = (int *)(malloc(__body_i_19__ * sizeof(int )));
    int __nlocal_0__ = __body_0__;
    int *__lb_i_i_0__ = (int *)(malloc(__loop_i_0__ * sizeof(int )));
    int *__ub_i_i_0__ = (int *)(malloc(__loop_i_0__ * sizeof(int )));
    int *__lb_j_i_1__ = (int *)(malloc(__loop_i_1__ * sizeof(int )));
    int *__ub_j_i_1__ = (int *)(malloc(__loop_i_1__ * sizeof(int )));
    int *__lb_p_i_2__ = (int *)(malloc(__loop_i_2__ * sizeof(int )));
    int *__ub_p_i_2__ = (int *)(malloc(__loop_i_2__ * sizeof(int )));
    int *__lb_i_i_3__ = (int *)(malloc(__loop_i_3__ * sizeof(int )));
    int *__ub_i_i_3__ = (int *)(malloc(__loop_i_3__ * sizeof(int )));
    int *__lb_j_i_4__ = (int *)(malloc(__loop_i_4__ * sizeof(int )));
    int *__ub_j_i_4__ = (int *)(malloc(__loop_i_4__ * sizeof(int )));
    int *__lb_i_i_5__ = (int *)(malloc(__loop_i_5__ * sizeof(int )));
    int *__ub_i_i_5__ = (int *)(malloc(__loop_i_5__ * sizeof(int )));
    int *__lb_j_i_6__ = (int *)(malloc(__loop_i_6__ * sizeof(int )));
    int *__ub_j_i_6__ = (int *)(malloc(__loop_i_6__ * sizeof(int )));
    int *__lb_i_i_7__ = (int *)(malloc(__loop_i_7__ * sizeof(int )));
    int *__ub_i_i_7__ = (int *)(malloc(__loop_i_7__ * sizeof(int )));
    int *__lb_j_i_8__ = (int *)(malloc(__loop_i_8__ * sizeof(int )));
    int *__ub_j_i_8__ = (int *)(malloc(__loop_i_8__ * sizeof(int )));
    int *__if_cond_0__ = (int *)(malloc(sizeof(int ) * __if_0__));
    int __nlocal_1__ = __body_1__;
    int *__lb_i_i_9__ = (int *)(malloc(__loop_i_9__ * sizeof(int )));
    int *__ub_i_i_9__ = (int *)(malloc(__loop_i_9__ * sizeof(int )));
    int *__lb_j_i_10__ = (int *)(malloc(__loop_i_10__ * sizeof(int )));
    int *__ub_j_i_10__ = (int *)(malloc(__loop_i_10__ * sizeof(int )));
    int *__lb_p_i_11__ = (int *)(malloc(__loop_i_11__ * sizeof(int )));
    int *__ub_p_i_11__ = (int *)(malloc(__loop_i_11__ * sizeof(int )));
    int *__lb_i_i_12__ = (int *)(malloc(__loop_i_12__ * sizeof(int )));
    int *__ub_i_i_12__ = (int *)(malloc(__loop_i_12__ * sizeof(int )));
    int *__lb_j_i_13__ = (int *)(malloc(__loop_i_13__ * sizeof(int )));
    int *__ub_j_i_13__ = (int *)(malloc(__loop_i_13__ * sizeof(int )));
    int *__lb_i_i_14__ = (int *)(malloc(__loop_i_14__ * sizeof(int )));
    int *__ub_i_i_14__ = (int *)(malloc(__loop_i_14__ * sizeof(int )));
    int *__lb_j_i_15__ = (int *)(malloc(__loop_i_15__ * sizeof(int )));
    int *__ub_j_i_15__ = (int *)(malloc(__loop_i_15__ * sizeof(int )));
    int *__lb_i_i_16__ = (int *)(malloc(__loop_i_16__ * sizeof(int )));
    int *__ub_i_i_16__ = (int *)(malloc(__loop_i_16__ * sizeof(int )));
    int *__lb_j_i_17__ = (int *)(malloc(__loop_i_17__ * sizeof(int )));
    int *__ub_j_i_17__ = (int *)(malloc(__loop_i_17__ * sizeof(int )));
    int *__lb_i_i_18__ = (int *)(malloc(__loop_i_18__ * sizeof(int )));
    int *__ub_i_i_18__ = (int *)(malloc(__loop_i_18__ * sizeof(int )));
    int *__lb_j_i_19__ = (int *)(malloc(__loop_i_19__ * sizeof(int )));
    int *__ub_j_i_19__ = (int *)(malloc(__loop_i_19__ * sizeof(int )));
    int *__if_cond_2__ = (int *)(malloc(sizeof(int ) * __if_2__));
    int __nlocal_2__ = __body_2__;
    int *__lb_i_i_20__ = (int *)(malloc(__loop_i_20__ * sizeof(int )));
    int *__ub_i_i_20__ = (int *)(malloc(__loop_i_20__ * sizeof(int )));
    int *__lb_j_i_21__ = (int *)(malloc(__loop_i_21__ * sizeof(int )));
    int *__ub_j_i_21__ = (int *)(malloc(__loop_i_21__ * sizeof(int )));
    int *__lb_p_i_22__ = (int *)(malloc(__loop_i_22__ * sizeof(int )));
    int *__ub_p_i_22__ = (int *)(malloc(__loop_i_22__ * sizeof(int )));
    int *__lb_i_i_23__ = (int *)(malloc(__loop_i_23__ * sizeof(int )));
    int *__ub_i_i_23__ = (int *)(malloc(__loop_i_23__ * sizeof(int )));
    int *__lb_j_i_24__ = (int *)(malloc(__loop_i_24__ * sizeof(int )));
    int *__ub_j_i_24__ = (int *)(malloc(__loop_i_24__ * sizeof(int )));
    int *__lb_i_i_25__ = (int *)(malloc(__loop_i_25__ * sizeof(int )));
    int *__ub_i_i_25__ = (int *)(malloc(__loop_i_25__ * sizeof(int )));
    int *__lb_j_i_26__ = (int *)(malloc(__loop_i_26__ * sizeof(int )));
    int *__ub_j_i_26__ = (int *)(malloc(__loop_i_26__ * sizeof(int )));
    int *__lb_i_i_27__ = (int *)(malloc(__loop_i_27__ * sizeof(int )));
    int *__ub_i_i_27__ = (int *)(malloc(__loop_i_27__ * sizeof(int )));
    int *__lb_j_i_28__ = (int *)(malloc(__loop_i_28__ * sizeof(int )));
    int *__ub_j_i_28__ = (int *)(malloc(__loop_i_28__ * sizeof(int )));
    int *__if_cond_4__ = (int *)(malloc(sizeof(int ) * __if_4__));
    int __nlocal_3__ = __body_3__;
    int *__lb_i_i_29__ = (int *)(malloc(__loop_i_29__ * sizeof(int )));
    int *__ub_i_i_29__ = (int *)(malloc(__loop_i_29__ * sizeof(int )));
    int *__lb_j_i_30__ = (int *)(malloc(__loop_i_30__ * sizeof(int )));
    int *__ub_j_i_30__ = (int *)(malloc(__loop_i_30__ * sizeof(int )));
    __body_0__ = 0;
    __loop_i_0__ = 0;
    __body_i_0__ = 0;
    __loop_i_1__ = 0;
    __body_i_1__ = 0;
    __loop_i_2__ = 0;
    __body_i_2__ = 0;
    __loop_i_3__ = 0;
    __body_i_3__ = 0;
    __loop_i_4__ = 0;
    __body_i_4__ = 0;
    __loop_i_5__ = 0;
    __body_i_5__ = 0;
    __loop_i_6__ = 0;
    __body_i_6__ = 0;
    __loop_i_7__ = 0;
    __body_i_7__ = 0;
    __loop_i_8__ = 0;
    __body_i_8__ = 0;
    __if_0__ = 0;
    __then_0__ = 0;
    __else_1__ = 0;
    __body_1__ = 0;
    __loop_i_9__ = 0;
    __body_i_9__ = 0;
    __loop_i_10__ = 0;
    __body_i_10__ = 0;
    __loop_i_11__ = 0;
    __body_i_11__ = 0;
    __loop_i_12__ = 0;
    __body_i_12__ = 0;
    __loop_i_13__ = 0;
    __body_i_13__ = 0;
    __loop_i_14__ = 0;
    __body_i_14__ = 0;
    __loop_i_15__ = 0;
    __body_i_15__ = 0;
    __loop_i_16__ = 0;
    __body_i_16__ = 0;
    __loop_i_17__ = 0;
    __body_i_17__ = 0;
    __loop_i_18__ = 0;
    __body_i_18__ = 0;
    __loop_i_19__ = 0;
    __body_i_19__ = 0;
    __if_2__ = 0;
    __then_2__ = 0;
    __else_3__ = 0;
    __body_2__ = 0;
    __loop_i_20__ = 0;
    __body_i_20__ = 0;
    __loop_i_21__ = 0;
    __body_i_21__ = 0;
    __loop_i_22__ = 0;
    __body_i_22__ = 0;
    __loop_i_23__ = 0;
    __body_i_23__ = 0;
    __loop_i_24__ = 0;
    __body_i_24__ = 0;
    __loop_i_25__ = 0;
    __body_i_25__ = 0;
    __loop_i_26__ = 0;
    __body_i_26__ = 0;
    __loop_i_27__ = 0;
    __body_i_27__ = 0;
    __loop_i_28__ = 0;
    __body_i_28__ = 0;
    __if_4__ = 0;
    __then_4__ = 0;
    __else_5__ = 0;
    __body_3__ = 0;
    __loop_i_29__ = 0;
    __body_i_29__ = 0;
    __loop_i_30__ = 0;
    __body_i_30__ = 0;
    for (k = 0; k < nf; k++) {
      if (get_vertex_home(0,k) == __myid__) {{
          __lb_i_i_0__[__loop_i_0__] = 0;
          __ub_i_i_0__[__loop_i_0__] = get_elem(6,get_elem(1,k));
          for (i = 0; i < get_elem(6,get_elem(1,k)); i++) {
            __lb_j_i_1__[__loop_i_1__] = 0;
            __ub_j_i_1__[__loop_i_1__] = get_elem(7,get_elem(1,k));
            for (j = 0; j < get_elem(7,get_elem(1,k)); j++) {
              if (j == 0) {
                __o_phi_old_j_0_[__loop_i_1__] = (get_elem(9,get_elem(1,k)) + (i * get_elem(7,get_elem(1,k)))) + j;
		__trace_phi_f_t_0__[__loop_i_1__] = (i+2)*(max_f_x+4)+j+2;
	      }
              else ;
              __body_i_1__ = __body_i_1__ + 1;;
            }
            __loop_i_1__ = __loop_i_1__ + 1;
            __body_i_0__ = __body_i_0__ + 1;;
          }
          __loop_i_0__ = __loop_i_0__ + 1;
          __lb_p_i_2__[__loop_i_2__] = get_elem(10,get_elem(1,k));
          __ub_p_i_2__[__loop_i_2__] = get_elem(10,get_elem(1,k) + 1);
          for (p = get_elem(10,get_elem(1,k)); p < get_elem(10,get_elem(1,k) + 1); p++) {{
              __if_cond_0__[__if_0__] = get_elem(16,p) == 1;
              if (__if_cond_0__[__if_0__]) {
                __lb_i_i_3__[__loop_i_3__] = get_elem(11,p);
                __ub_i_i_3__[__loop_i_3__] = get_elem(12,p);
                for (i = get_elem(11,p); i < get_elem(12,p); i++) {
                  __lb_j_i_4__[__loop_i_4__] = get_elem(13,p);
                  __ub_j_i_4__[__loop_i_4__] = get_elem(14,p);
                  for (j = get_elem(13,p); j < get_elem(14,p); j++) {{
                      __i_phi_old_j_1_[__body_i_4__] = ((get_elem(9,get_elem(15,p)) + (((i * 2) - (get_elem(2,get_elem(15,p)) * 2)) * get_elem(7,get_elem(15,p)))) + (j * 2)) - (get_elem(4,get_elem(15,p)) * 2);
                      __i_phi_old_j_2_[__body_i_4__] = ((get_elem(9,get_elem(15,p)) + ((((i * 2) + 1) - (get_elem(2,get_elem(15,p)) * 2)) * get_elem(7,get_elem(15,p)))) + (j * 2)) - (get_elem(4,get_elem(15,p)) * 2);
                      __i_phi_old_j_3_[__body_i_4__] = (((get_elem(9,get_elem(15,p)) + (((i * 2) - (get_elem(2,get_elem(15,p)) * 2)) * get_elem(7,get_elem(15,p)))) + (j * 2)) + 1) - (get_elem(4,get_elem(15,p)) * 2);
                      __i_phi_old_j_4_[__body_i_4__] = (((get_elem(9,get_elem(15,p)) + ((((i * 2) + 1) - (get_elem(2,get_elem(15,p)) * 2)) * get_elem(7,get_elem(15,p)))) + (j * 2)) + 1) - (get_elem(4,get_elem(15,p)) * 2);;
		      __trace_phi_f_t_1__[__body_i_4__] = (2*i+2-get_elem(2,get_elem(1,k))*2)*(max_f_x+4) + (2*j+2-get_elem(4,get_elem(1,k))*2);
		      __trace_phi_f_t_2__[__body_i_4__] = (2*i+3-get_elem(2,get_elem(1,k))*2)*(max_f_x+4) + (2*j+2-get_elem(4,get_elem(1,k))*2);
		      __trace_phi_f_t_3__[__body_i_4__] = (2*i+2-get_elem(2,get_elem(1,k))*2)*(max_f_x+4) + (2*j+3-get_elem(4,get_elem(1,k))*2);
		      __trace_phi_f_t_4__[__body_i_4__] = (2*i+3-get_elem(2,get_elem(1,k))*2)*(max_f_x+4) + (2*j+3-get_elem(4,get_elem(1,k))*2);
                    }
                    __body_i_4__ = __body_i_4__ + 1;;
                  }
                  __loop_i_4__ = __loop_i_4__ + 1;
                  __body_i_3__ = __body_i_3__ + 1;;
                }
                __loop_i_3__ = __loop_i_3__ + 1;
                __then_0__ = __then_0__ + 1;;
              }
              else {
                __lb_i_i_5__[__loop_i_5__] = get_elem(11,p);
                __ub_i_i_5__[__loop_i_5__] = get_elem(12,p);
                for (i = get_elem(11,p); i < get_elem(12,p); i++) {
                  __lb_j_i_6__[__loop_i_6__] = get_elem(13,p);
                  __ub_j_i_6__[__loop_i_6__] = get_elem(14,p);
                  for (j = get_elem(13,p); j < get_elem(14,p); j++) {{
                      __i_phi_old_j_5_[__body_i_6__] = ((get_elem(9,get_elem(15,p)) + ((i - get_elem(2,get_elem(15,p))) * get_elem(7,get_elem(15,p)))) + j) - get_elem(4,get_elem(15,p));;
		      __trace_phi_f_t_5__[__body_i_6__] = (2*i+2-get_elem(2,get_elem(1,k))*2)*(max_f_x+4) + (2*j+2-get_elem(4,get_elem(1,k))*2);
		      __trace_phi_f_t_6__[__body_i_6__] = (2*i+3-get_elem(2,get_elem(1,k))*2)*(max_f_x+4) + (2*j+2-get_elem(4,get_elem(1,k))*2);
		      __trace_phi_f_t_7__[__body_i_6__] = (2*i+2-get_elem(2,get_elem(1,k))*2)*(max_f_x+4) + (2*j+3-get_elem(4,get_elem(1,k))*2);
		      __trace_phi_f_t_8__[__body_i_6__] = (2*i+3-get_elem(2,get_elem(1,k))*2)*(max_f_x+4) + (2*j+3-get_elem(4,get_elem(1,k))*2);
                    }
                    __body_i_6__ = __body_i_6__ + 1;;
                  }
                  __loop_i_6__ = __loop_i_6__ + 1;
                  __body_i_5__ = __body_i_5__ + 1;;
                }
                __loop_i_5__ = __loop_i_5__ + 1;
                __else_1__ = __else_1__ + 1;;
              }
              __if_0__ = __if_0__ + 1;;
            }
            __body_i_2__ = __body_i_2__ + 1;;
          }
          __loop_i_2__ = __loop_i_2__ + 1;
          __lb_i_i_7__[__loop_i_7__] = 0;
          __ub_i_i_7__[__loop_i_7__] = get_elem(6,get_elem(1,k));
          for (i = 0; i < get_elem(6,get_elem(1,k)); i++) {
            __lb_j_i_8__[__loop_i_8__] = 0;
            __ub_j_i_8__[__loop_i_8__] = get_elem(7,get_elem(1,k));
            for (j = 0; j < get_elem(7,get_elem(1,k)); j++) {
              if (j == 0) {
                __o_phi_j_6_[__loop_i_8__] = (get_elem(9,get_elem(1,k)) + (i * get_elem(7,get_elem(1,k)))) + j;
		__trace_phi_f_t_9__[__loop_i_8__] = (i+1)*(max_f_x+4)+(j+2);
		__trace_phi_f_t_10__[__loop_i_8__] = (i+2)*(max_f_x+4)+(j+1);
		__trace_phi_f_t_11__[__loop_i_8__] = (i+2)*(max_f_x+4)+(j+3);
		__trace_phi_f_t_12__[__loop_i_8__] = (i+3)*(max_f_x+4)+(j+2);
		__trace_phi_f_t_13__[__loop_i_8__] = (i+2)*(max_f_x+4)+(j+2);
	      }
              else ;
              if (j == 0)
                __o_rho_j_7_[__loop_i_8__] = (get_elem(9,get_elem(1,k)) + (i * get_elem(7,get_elem(1,k)))) + j;
              else ;
              __body_i_8__ = __body_i_8__ + 1;;
            }
            __loop_i_8__ = __loop_i_8__ + 1;
            __body_i_7__ = __body_i_7__ + 1;;
          }
          __loop_i_7__ = __loop_i_7__ + 1;;
        }
        __body_0__ = __body_0__ + 1;;
      }
      else ;;
    }
    for (k = 0; k < nf; k++) {
      if (get_vertex_home(0,k) == __myid__) {{
          __lb_i_i_9__[__loop_i_9__] = 0;
          __ub_i_i_9__[__loop_i_9__] = get_elem(6,get_elem(1,k));
          for (i = 0; i < get_elem(6,get_elem(1,k)); i++) {
            __lb_j_i_10__[__loop_i_10__] = 0;
            __ub_j_i_10__[__loop_i_10__] = get_elem(7,get_elem(1,k));
            for (j = 0; j < get_elem(7,get_elem(1,k)); j++) {
              if (j == 0) {
                __o_phi_j_8_[__loop_i_10__] = (get_elem(9,get_elem(1,k)) + (i * get_elem(7,get_elem(1,k)))) + j;
		__trace_phi_f_t_14__[__loop_i_10__] = (i+2)*(max_f_x+4)+(j+2);
	      }
              else ;
              __body_i_10__ = __body_i_10__ + 1;;
            }
            __loop_i_10__ = __loop_i_10__ + 1;
            __body_i_9__ = __body_i_9__ + 1;;
          }
          __loop_i_9__ = __loop_i_9__ + 1;
          __lb_p_i_11__[__loop_i_11__] = get_elem(10,get_elem(1,k));
          __ub_p_i_11__[__loop_i_11__] = get_elem(10,get_elem(1,k) + 1);
          for (p = get_elem(10,get_elem(1,k)); p < get_elem(10,get_elem(1,k) + 1); p++) {{
              __if_cond_2__[__if_2__] = get_elem(16,p) == 1;
              if (__if_cond_2__[__if_2__]) {
                __lb_i_i_12__[__loop_i_12__] = get_elem(11,p);
                __ub_i_i_12__[__loop_i_12__] = get_elem(12,p);
                for (i = get_elem(11,p); i < get_elem(12,p); i++) {
                  __lb_j_i_13__[__loop_i_13__] = get_elem(13,p);
                  __ub_j_i_13__[__loop_i_13__] = get_elem(14,p);
                  for (j = get_elem(13,p); j < get_elem(14,p); j++) {{
                      __i_phi_j_9_[__body_i_13__] = ((get_elem(9,get_elem(15,p)) + (((i * 2) - (get_elem(2,get_elem(15,p)) * 2)) * get_elem(7,get_elem(15,p)))) + (j * 2)) - (get_elem(4,get_elem(15,p)) * 2);
                      __i_phi_j_10_[__body_i_13__] = ((get_elem(9,get_elem(15,p)) + ((((i * 2) - (get_elem(2,get_elem(15,p)) * 2)) + 1) * get_elem(7,get_elem(15,p)))) + (j * 2)) - (get_elem(4,get_elem(15,p)) * 2);
                      __i_phi_j_11_[__body_i_13__] = (((get_elem(9,get_elem(15,p)) + (((i * 2) - (get_elem(2,get_elem(15,p)) * 2)) * get_elem(7,get_elem(15,p)))) + (j * 2)) - (get_elem(4,get_elem(15,p)) * 2)) + 1;
                      __i_phi_j_12_[__body_i_13__] = (((get_elem(9,get_elem(15,p)) + ((((i * 2) - (get_elem(2,get_elem(15,p)) * 2)) + 1) * get_elem(7,get_elem(15,p)))) + (j * 2)) - (get_elem(4,get_elem(15,p)) * 2)) + 1;;
		      __trace_phi_f_t_15__[__body_i_13__] = (i*2+2-get_elem(2,get_elem(1,k))*2)*(max_f_x+4)+(j*2+2-get_elem(4,get_elem(1,k))*2);
		      __trace_phi_f_t_16__[__body_i_13__] = (i*2+3-get_elem(2,get_elem(1,k))*2)*(max_f_x+4)+(j*2+2-get_elem(4,get_elem(1,k))*2);
		      __trace_phi_f_t_17__[__body_i_13__] = (i*2+2-get_elem(2,get_elem(1,k))*2)*(max_f_x+4)+(j*2+3-get_elem(4,get_elem(1,k))*2);
		      __trace_phi_f_t_18__[__body_i_13__] = (i*2+3-get_elem(2,get_elem(1,k))*2)*(max_f_x+4)+(j*2+3-get_elem(4,get_elem(1,k))*2);
                    }
                    __body_i_13__ = __body_i_13__ + 1;;
                  }
                  __loop_i_13__ = __loop_i_13__ + 1;
                  __body_i_12__ = __body_i_12__ + 1;;
                }
                __loop_i_12__ = __loop_i_12__ + 1;
                __then_2__ = __then_2__ + 1;;
              }
              else {
                __lb_i_i_14__[__loop_i_14__] = get_elem(11,p);
                __ub_i_i_14__[__loop_i_14__] = get_elem(12,p);
                for (i = get_elem(11,p); i < get_elem(12,p); i++) {
                  __lb_j_i_15__[__loop_i_15__] = get_elem(13,p);
                  __ub_j_i_15__[__loop_i_15__] = get_elem(14,p);
                  for (j = get_elem(13,p); j < get_elem(14,p); j++) {{
                      __i_phi_j_13_[__body_i_15__] = ((get_elem(9,get_elem(15,p)) + ((i - get_elem(2,get_elem(15,p))) * get_elem(7,get_elem(15,p)))) + j) - get_elem(4,get_elem(15,p));;
		      __trace_phi_f_t_19__[__body_i_15__] = (i*2+2-get_elem(2,get_elem(1,k))*2)*(max_f_x+4)+(j*2+2-get_elem(4,get_elem(1,k))*2);
		      __trace_phi_f_t_20__[__body_i_15__] = (i*2+3-get_elem(2,get_elem(1,k))*2)*(max_f_x+4)+(j*2+2-get_elem(4,get_elem(1,k))*2);
		      __trace_phi_f_t_21__[__body_i_15__] = (i*2+2-get_elem(2,get_elem(1,k))*2)*(max_f_x+4)+(j*2+3-get_elem(4,get_elem(1,k))*2);
		      __trace_phi_f_t_22__[__body_i_15__] = (i*2+3-get_elem(2,get_elem(1,k))*2)*(max_f_x+4)+(j*2+3-get_elem(4,get_elem(1,k))*2);
                    }
                    __body_i_15__ = __body_i_15__ + 1;;
                  }
                  __loop_i_15__ = __loop_i_15__ + 1;
                  __body_i_14__ = __body_i_14__ + 1;;
                }
                __loop_i_14__ = __loop_i_14__ + 1;
                __else_3__ = __else_3__ + 1;;
              }
              __if_2__ = __if_2__ + 1;;
            }
            __body_i_11__ = __body_i_11__ + 1;;
          }
          __loop_i_11__ = __loop_i_11__ + 1;
          __lb_i_i_16__[__loop_i_16__] = 0;
          __ub_i_i_16__[__loop_i_16__] = get_elem(6,get_elem(1,k));
          for (i = 0; i < get_elem(6,get_elem(1,k)); i++) {
            __lb_j_i_17__[__loop_i_17__] = 0;
            __ub_j_i_17__[__loop_i_17__] = get_elem(7,get_elem(1,k));
            for (j = 0; j < get_elem(7,get_elem(1,k)); j++) {
              if (j == 0) {
                __o_rho_j_14_[__loop_i_17__] = (get_elem(9,get_elem(1,k)) + (i * get_elem(7,get_elem(1,k)))) + j;
		__trace_residual_0__[__loop_i_17__] = i*max_f_x+j;
		__trace_phi_f_t_23__[__loop_i_17__] = (i+1)*(max_f_x+4)+j+2;
		__trace_phi_f_t_24__[__loop_i_17__] = (i+2)*(max_f_x+4)+j+1;
		__trace_phi_f_t_25__[__loop_i_17__] = (i+2)*(max_f_x+4)+j+3;
		__trace_phi_f_t_26__[__loop_i_17__] = (i+3)*(max_f_x+4)+j+2;
		__trace_phi_f_t_27__[__loop_i_17__] = (i+2)*(max_f_x+4)+j+2;
	      }
              else ;
              __body_i_17__ = __body_i_17__ + 1;;
            }
            __loop_i_17__ = __loop_i_17__ + 1;
            __body_i_16__ = __body_i_16__ + 1;;
          }
          __loop_i_16__ = __loop_i_16__ + 1;
          __lb_i_i_18__[__loop_i_18__] = get_elem(2,get_elem(1,k));
          __ub_i_i_18__[__loop_i_18__] = get_elem(3,get_elem(1,k));
          for (i = get_elem(2,get_elem(1,k)); i < get_elem(3,get_elem(1,k)); i++) {
            __lb_j_i_19__[__loop_i_19__] = get_elem(4,get_elem(1,k));
            __ub_j_i_19__[__loop_i_19__] = get_elem(5,get_elem(1,k));
            for (j = get_elem(4,get_elem(1,k)); j < get_elem(5,get_elem(1,k)); j++) {{
                __i_rho_write_j_15_[__body_i_19__] = (get_elem(9,get_elem(8,k)) + ((i - get_elem(2,get_elem(8,k))) * get_elem(7,get_elem(8,k)))) + (j - get_elem(4,get_elem(8,k)));
                __i_phi_old_j_16_[__body_i_19__] = (get_elem(9,get_elem(8,k)) + ((i - get_elem(2,get_elem(8,k))) * get_elem(7,get_elem(8,k)))) + (j - get_elem(4,get_elem(8,k)));;
		__trace_residual_1__[__body_i_19__] = 2*(i-get_elem(2,get_elem(1,k)))*max_f_x+2*(j-get_elem(4,get_elem(1,k)));
		__trace_residual_2__[__body_i_19__] = (2*(i-get_elem(2,get_elem(1,k)))+1)*max_f_x+2*(j-get_elem(4,get_elem(1,k)));
		__trace_residual_3__[__body_i_19__] = 2*(i-get_elem(2,get_elem(1,k)))*max_f_x+2*(j-get_elem(4,get_elem(1,k)))+1;
		__trace_residual_4__[__body_i_19__] = (2*(i-get_elem(2,get_elem(1,k)))+1)*max_f_x+2*(j-get_elem(4,get_elem(1,k)))+1;
              }
              __body_i_19__ = __body_i_19__ + 1;;
            }
            __loop_i_19__ = __loop_i_19__ + 1;
            __body_i_18__ = __body_i_18__ + 1;;
          }
          __loop_i_18__ = __loop_i_18__ + 1;;
        }
        __body_1__ = __body_1__ + 1;;
      }
      else ;;
    }
    for (k = 0; k < nc; k++) {
      if (get_vertex_home(1,k) == __myid__) {{
          __lb_i_i_20__[__loop_i_20__] = 0;
          __ub_i_i_20__[__loop_i_20__] = get_elem(6,get_elem(0,k));
          for (i = 0; i < get_elem(6,get_elem(0,k)); i++) {
            __lb_j_i_21__[__loop_i_21__] = 0;
            __ub_j_i_21__[__loop_i_21__] = get_elem(7,get_elem(0,k));
            for (j = 0; j < get_elem(7,get_elem(0,k)); j++) {
              if (j == 0) {
                __o_phi_old_j_17_[__loop_i_21__] = (get_elem(9,get_elem(0,k)) + (i * get_elem(7,get_elem(0,k)))) + j;
		__trace_phi_c_t_0__[__loop_i_21__] = (i+1)*(max_c_x+2)+j+1;
	      }
              else ;
              __body_i_21__ = __body_i_21__ + 1;;
            }
            __loop_i_21__ = __loop_i_21__ + 1;
            __body_i_20__ = __body_i_20__ + 1;;
          }
          __loop_i_20__ = __loop_i_20__ + 1;
          __lb_p_i_22__[__loop_i_22__] = get_elem(10,get_elem(0,k));
          __ub_p_i_22__[__loop_i_22__] = get_elem(10,get_elem(0,k) + 1);
          for (p = get_elem(10,get_elem(0,k)); p < get_elem(10,get_elem(0,k) + 1); p++) {
            __if_cond_4__[__if_4__] = get_elem(16,p) == -1;
            if (__if_cond_4__[__if_4__]) {{
                __lb_i_i_23__[__loop_i_23__] = get_elem(11,p);
                __ub_i_i_23__[__loop_i_23__] = get_elem(12,p);
                for (i = get_elem(11,p); i < get_elem(12,p); i++) {
                  __lb_j_i_24__[__loop_i_24__] = get_elem(13,p);
                  __ub_j_i_24__[__loop_i_24__] = get_elem(14,p);
                  for (j = get_elem(13,p); j < get_elem(14,p); j++) {
		    if( j == get_elem(13,p))
		      __trace_phi_c_t_1__[__loop_i_24__] = (i+1-get_elem(2,get_elem(0,k)))*(max_c_x+2) + (j+1-get_elem(4,get_elem(0,k)));
                    __body_i_24__ = __body_i_24__ + 1;;
                  }
                  __loop_i_24__ = __loop_i_24__ + 1;
                  __body_i_23__ = __body_i_23__ + 1;;
                }
                __loop_i_23__ = __loop_i_23__ + 1;;
              }
              __then_4__ = __then_4__ + 1;;
            }
            else {{
                __lb_i_i_25__[__loop_i_25__] = get_elem(11,p);
                __ub_i_i_25__[__loop_i_25__] = get_elem(12,p);
                for (i = get_elem(11,p); i < get_elem(12,p); i++) {
                  __lb_j_i_26__[__loop_i_26__] = get_elem(13,p);
                  __ub_j_i_26__[__loop_i_26__] = get_elem(14,p);
                  for (j = get_elem(13,p); j < get_elem(14,p); j++) {
		    if( j == get_elem(13,p) ){
		      __o_phi_old_j_18_[__loop_i_26__] = ((get_elem(9,get_elem(15,p)) + ((i - get_elem(2,get_elem(15,p))) * get_elem(7,get_elem(15,p)))) + j) - get_elem(4,get_elem(15,p));
		      //printf("Box:%d,index:%d\n",__o_phi_old_j_18_);
		      __trace_phi_c_t_2__[__loop_i_26__] = (i+1-get_elem(2,get_elem(0,k)))*(max_c_x+2) + (j+1-get_elem(4,get_elem(0,k)));
		    
		    }
                    __body_i_26__ = __body_i_26__ + 1;;
                  }
                  __loop_i_26__ = __loop_i_26__ + 1;
                  __body_i_25__ = __body_i_25__ + 1;;
                }
                __loop_i_25__ = __loop_i_25__ + 1;;
              }
              __else_5__ = __else_5__ + 1;;
            }
            __if_4__ = __if_4__ + 1;
            __body_i_22__ = __body_i_22__ + 1;;
          }
          __loop_i_22__ = __loop_i_22__ + 1;
          __lb_i_i_27__[__loop_i_27__] = 0;
          __ub_i_i_27__[__loop_i_27__] = get_elem(6,get_elem(0,k));
          for (i = 0; i < get_elem(6,get_elem(0,k)); i++) {
            __lb_j_i_28__[__loop_i_28__] = 0;
            __ub_j_i_28__[__loop_i_28__] = get_elem(7,get_elem(0,k));
            for (j = 0; j < get_elem(7,get_elem(0,k)); j++) {
              if (j == 0) {
                __o_phi_j_19_[__loop_i_28__] = (get_elem(9,get_elem(0,k)) + (i * get_elem(7,get_elem(0,k)))) + j;
		__trace_phi_c_t_3__[__loop_i_28__] = (i)*(max_c_x+2)+j+1;
		__trace_phi_c_t_4__[__loop_i_28__] = (i+1)*(max_c_x+2)+j;
		__trace_phi_c_t_5__[__loop_i_28__] = (i+1)*(max_c_x+2)+j+2;
		__trace_phi_c_t_6__[__loop_i_28__] = (i+2)*(max_c_x+2)+j+1;
		__trace_phi_c_t_7__[__loop_i_28__] = (i+1)*(max_c_x+2)+j+1;
	      }
              else ;
              if (j == 0)
                __o_rho_j_20_[__loop_i_28__] = (get_elem(9,get_elem(0,k)) + (i * get_elem(7,get_elem(0,k)))) + j;
              else ;
              __body_i_28__ = __body_i_28__ + 1;;
            }
            __loop_i_28__ = __loop_i_28__ + 1;
            __body_i_27__ = __body_i_27__ + 1;;
          }
          __loop_i_27__ = __loop_i_27__ + 1;;
        }
        __body_2__ = __body_2__ + 1;;
      }
      else ;;
    }
    for (k = 0; k < nf; k++) {
      if (get_vertex_home(0,k) == __myid__) {{
          __lb_i_i_29__[__loop_i_29__] = get_elem(2,get_elem(1,k));
          __ub_i_i_29__[__loop_i_29__] = get_elem(3,get_elem(1,k));
          for (i = get_elem(2,get_elem(1,k)); i < get_elem(3,get_elem(1,k)); i++) {
            __lb_j_i_30__[__loop_i_30__] = get_elem(4,get_elem(1,k));
            __ub_j_i_30__[__loop_i_30__] = get_elem(5,get_elem(1,k));
            for (j = get_elem(4,get_elem(1,k)); j < get_elem(5,get_elem(1,k)); j++) {{
                __i_phi_old_j_21_[__body_i_30__] = (get_elem(9,get_elem(1,k)) + (((i - get_elem(2,get_elem(1,k))) * 2) * get_elem(7,get_elem(1,k)))) + ((j - get_elem(4,get_elem(1,k))) * 2);
                __i_phi_j_22_[__body_i_30__] = ((get_elem(9,get_elem(8,k)) + ((i - get_elem(2,get_elem(8,k))) * get_elem(7,get_elem(8,k)))) + j) - get_elem(4,get_elem(8,k));
                __i_phi_old_j_23_[__body_i_30__] = (get_elem(9,get_elem(1,k)) + ((((i - get_elem(2,get_elem(1,k))) * 2) + 1) * get_elem(7,get_elem(1,k)))) + ((j - get_elem(4,get_elem(1,k))) * 2);
                __i_phi_old_j_24_[__body_i_30__] = ((get_elem(9,get_elem(1,k)) + (((i - get_elem(2,get_elem(1,k))) * 2) * get_elem(7,get_elem(1,k)))) + ((j - get_elem(4,get_elem(1,k))) * 2)) + 1;
                __i_phi_old_j_25_[__body_i_30__] = ((get_elem(9,get_elem(1,k)) + ((((i - get_elem(2,get_elem(1,k))) * 2) + 1) * get_elem(7,get_elem(1,k)))) + ((j - get_elem(4,get_elem(1,k))) * 2)) + 1;;
              }
              __body_i_30__ = __body_i_30__ + 1;;
            }
            __loop_i_30__ = __loop_i_30__ + 1;
            __body_i_29__ = __body_i_29__ + 1;;
          }
          __loop_i_29__ = __loop_i_29__ + 1;;
        }
        __body_3__ = __body_3__ + 1;;
      }
      else ;;
    }
    int __localsize_phi__ = get_local_size(__thread_id__,0);
    double *__local_phi__ = (double *)(malloc(__localsize_phi__ *  1  * sizeof(double )));
    populate_local_array(__thread_id__,0,((double *)__local_phi__),((double *)phi), 1 );
    renumber_offset_array(__thread_id__,0,__loop_i_8__,__o_phi_j_6_,__lb_j_i_8__);
    renumber_offset_array(__thread_id__,0,__loop_i_10__,__o_phi_j_8_,__lb_j_i_10__);
    renumber_access_array(__thread_id__,0,__body_i_13__,__i_phi_j_9_);
    renumber_access_array(__thread_id__,0,__body_i_13__,__i_phi_j_10_);
    renumber_access_array(__thread_id__,0,__body_i_13__,__i_phi_j_11_);
    renumber_access_array(__thread_id__,0,__body_i_13__,__i_phi_j_12_);
    renumber_access_array(__thread_id__,0,__body_i_15__,__i_phi_j_13_);
    renumber_offset_array(__thread_id__,0,__loop_i_28__,__o_phi_j_19_,__lb_j_i_28__);
    renumber_access_array(__thread_id__,0,__body_i_30__,__i_phi_j_22_);
    communicate_writes_for(__thread_id__,0,0);
    communicate_writes_for(__thread_id__,2,0);
    communicate_reads_for(__thread_id__,1,0);
    communicate_reads_for(__thread_id__,3,0);
    int __localsize_phi_old__ = get_local_size(__thread_id__,1);
    double *__local_phi_old__ = (double *)(malloc(__localsize_phi_old__ *  1  * sizeof(double )));
    populate_local_array(__thread_id__,1,((double *)__local_phi_old__),((double *)phi_old), 1 );
    renumber_offset_array(__thread_id__,1,__loop_i_1__,__o_phi_old_j_0_,__lb_j_i_1__);
    renumber_access_array(__thread_id__,1,__body_i_4__,__i_phi_old_j_1_);
    renumber_access_array(__thread_id__,1,__body_i_4__,__i_phi_old_j_2_);
    renumber_access_array(__thread_id__,1,__body_i_4__,__i_phi_old_j_3_);
    renumber_access_array(__thread_id__,1,__body_i_4__,__i_phi_old_j_4_);
    renumber_access_array(__thread_id__,1,__body_i_6__,__i_phi_old_j_5_);
    renumber_access_array(__thread_id__,1,__body_i_19__,__i_phi_old_j_16_);
    renumber_offset_array(__thread_id__,1,__loop_i_21__,__o_phi_old_j_17_,__lb_j_i_21__);
    renumber_offset_array(__thread_id__,1,__loop_i_26__,__o_phi_old_j_18_,__lb_j_i_26__);
    renumber_access_array(__thread_id__,1,__body_i_30__,__i_phi_old_j_21_);
    renumber_access_array(__thread_id__,1,__body_i_30__,__i_phi_old_j_23_);
    renumber_access_array(__thread_id__,1,__body_i_30__,__i_phi_old_j_24_);
    renumber_access_array(__thread_id__,1,__body_i_30__,__i_phi_old_j_25_);
    communicate_writes_for(__thread_id__,1,1);
    communicate_writes_for(__thread_id__,3,1);
    communicate_reads_for(__thread_id__,0,1);
    communicate_reads_for(__thread_id__,2,1);
    int __localsize_rho__ = get_local_size(__thread_id__,2);
    double *__local_rho__ = (double *)(malloc(__localsize_rho__ *  1  * sizeof(double )));
    populate_local_array(__thread_id__,2,((double *)__local_rho__),((double *)rho), 1 );
    renumber_offset_array(__thread_id__,2,__loop_i_8__,__o_rho_j_7_,__lb_j_i_8__);
    renumber_offset_array(__thread_id__,2,__loop_i_17__,__o_rho_j_14_,__lb_j_i_17__);
    renumber_offset_array(__thread_id__,2,__loop_i_28__,__o_rho_j_20_,__lb_j_i_28__);
    renumber_access_array(__thread_id__,2,__body_i_19__,__i_rho_write_j_15_);
    communicate_reads_for(__thread_id__,2,2);
    communicate_writes_for(__thread_id__,1,2);
    setup_executor(__thread_id__);;;
    for (t = 0; t < 10; t++) {
      
#pragma part_loop
      __body_0__ = 0;
      __loop_i_0__ = 0;
      __body_i_0__ = 0;
      __loop_i_1__ = 0;
      __body_i_1__ = 0;
      __loop_i_2__ = 0;
      __body_i_2__ = 0;
      __loop_i_3__ = 0;
      __body_i_3__ = 0;
      __loop_i_4__ = 0;
      __body_i_4__ = 0;
      __loop_i_5__ = 0;
      __body_i_5__ = 0;
      __loop_i_6__ = 0;
      __body_i_6__ = 0;
      __loop_i_7__ = 0;
      __body_i_7__ = 0;
      __loop_i_8__ = 0;
      __body_i_8__ = 0;
      __if_0__ = 0;
      __then_0__ = 0;
      __else_1__ = 0;
      init_write_ghosts(__thread_id__,0);
      communicate_reads(__thread_id__,0);
      for (k = 0; k < __nlocal_0__; ++k) {
        for (i = __lb_i_i_0__[__loop_i_0__]; i < __ub_i_i_0__[__loop_i_0__]; ++i) {
          __o_phi_old_j_0__offset = __o_phi_old_j_0_[__loop_i_1__];
	  __trace_phi_f_t_0__offset = __trace_phi_f_t_0__[__loop_i_1__] - __lb_j_i_1__[__loop_i_1__];
          for (j = __lb_j_i_1__[__loop_i_1__]; j < __ub_j_i_1__[__loop_i_1__]; ++j) {
            phi_f_t[__trace_phi_f_t_0__offset+j] = __local_phi_old__[__o_phi_old_j_0__offset + j];
            __body_i_1__ = __body_i_1__ + 1;;
          }
          __loop_i_1__ = __loop_i_1__ + 1;
          __body_i_0__ = __body_i_0__ + 1;;
        }
        __loop_i_0__ = __loop_i_0__ + 1;
        for (p = __lb_p_i_2__[__loop_i_2__]; p < __ub_p_i_2__[__loop_i_2__]; ++p) {
          if (__if_cond_0__[__if_0__]) {
            for (i = __lb_i_i_3__[__loop_i_3__]; i < __ub_i_i_3__[__loop_i_3__]; ++i) {
              for (j = __lb_j_i_4__[__loop_i_4__]; j < __ub_j_i_4__[__loop_i_4__]; ++j) {
                phi_f_t[__trace_phi_f_t_1__[__body_i_4__]] = __local_phi_old__[__i_phi_old_j_1_[__body_i_4__]];
                phi_f_t[__trace_phi_f_t_2__[__body_i_4__]] = __local_phi_old__[__i_phi_old_j_2_[__body_i_4__]];
                phi_f_t[__trace_phi_f_t_3__[__body_i_4__]] = __local_phi_old__[__i_phi_old_j_3_[__body_i_4__]];
                phi_f_t[__trace_phi_f_t_4__[__body_i_4__]] = __local_phi_old__[__i_phi_old_j_4_[__body_i_4__]];
                __body_i_4__ = __body_i_4__ + 1;;
              }
              __loop_i_4__ = __loop_i_4__ + 1;
              __body_i_3__ = __body_i_3__ + 1;;
            }
            __loop_i_3__ = __loop_i_3__ + 1;
            __then_0__ = __then_0__ + 1;;
          }
          else {
            for (i = __lb_i_i_5__[__loop_i_5__]; i < __ub_i_i_5__[__loop_i_5__]; ++i) {
              for (j = __lb_j_i_6__[__loop_i_6__]; j < __ub_j_i_6__[__loop_i_6__]; ++j) {
                phi_f_t[__trace_phi_f_t_5__[__body_i_6__]] = __local_phi_old__[__i_phi_old_j_5_[__body_i_6__]];
                phi_f_t[__trace_phi_f_t_6__[__body_i_6__]] = __local_phi_old__[__i_phi_old_j_5_[__body_i_6__]];
                phi_f_t[__trace_phi_f_t_7__[__body_i_6__]] = __local_phi_old__[__i_phi_old_j_5_[__body_i_6__]];
                phi_f_t[__trace_phi_f_t_8__[__body_i_6__]] = __local_phi_old__[__i_phi_old_j_5_[__body_i_6__]];
                __body_i_6__ = __body_i_6__ + 1;;
              }
              __loop_i_6__ = __loop_i_6__ + 1;
              __body_i_5__ = __body_i_5__ + 1;;
            }
            __loop_i_5__ = __loop_i_5__ + 1;
            __else_1__ = __else_1__ + 1;;
          }
          __if_0__ = __if_0__ + 1;
          __body_i_2__ = __body_i_2__ + 1;;
        }
        __loop_i_2__ = __loop_i_2__ + 1;
        for (i = __lb_i_i_7__[__loop_i_7__]; i < __ub_i_i_7__[__loop_i_7__]; ++i) {
          __o_phi_j_6__offset = __o_phi_j_6_[__loop_i_8__];
          __o_rho_j_7__offset = __o_rho_j_7_[__loop_i_8__];
	  __trace_phi_f_t_9__offset = __trace_phi_f_t_9__[__loop_i_8__] - __lb_j_i_8__[__loop_i_8__];
	  __trace_phi_f_t_10__offset = __trace_phi_f_t_10__[__loop_i_8__] - __lb_j_i_8__[__loop_i_8__];
	  __trace_phi_f_t_11__offset = __trace_phi_f_t_11__[__loop_i_8__] - __lb_j_i_8__[__loop_i_8__];
	  __trace_phi_f_t_12__offset = __trace_phi_f_t_12__[__loop_i_8__] - __lb_j_i_8__[__loop_i_8__];
	  __trace_phi_f_t_13__offset = __trace_phi_f_t_13__[__loop_i_8__] - __lb_j_i_8__[__loop_i_8__];
          for (j = __lb_j_i_8__[__loop_i_8__]; j < __ub_j_i_8__[__loop_i_8__]; ++j) {
            __local_phi__[__o_phi_j_6__offset + j] = ((phi_f_t[__trace_phi_f_t_9__offset+j] + phi_f_t[__trace_phi_f_t_10__offset+j] + phi_f_t[__trace_phi_f_t_11__offset+j] + phi_f_t[__trace_phi_f_t_12__offset+j] - 4 * phi_f_t[__trace_phi_f_t_13__offset+j] -  __local_rho__[__o_rho_j_7__offset + j] * 0.25) / 8.0);
            __body_i_8__ = __body_i_8__ + 1;;
          }
          __loop_i_8__ = __loop_i_8__ + 1;
          __body_i_7__ = __body_i_7__ + 1;;
        }
        __loop_i_7__ = __loop_i_7__ + 1;;
      }
      communicate_writes(__thread_id__,0);
      
#pragma part_loop
      __body_1__ = 0;
      __loop_i_9__ = 0;
      __body_i_9__ = 0;
      __loop_i_10__ = 0;
      __body_i_10__ = 0;
      __loop_i_11__ = 0;
      __body_i_11__ = 0;
      __loop_i_12__ = 0;
      __body_i_12__ = 0;
      __loop_i_13__ = 0;
      __body_i_13__ = 0;
      __loop_i_14__ = 0;
      __body_i_14__ = 0;
      __loop_i_15__ = 0;
      __body_i_15__ = 0;
      __loop_i_16__ = 0;
      __body_i_16__ = 0;
      __loop_i_17__ = 0;
      __body_i_17__ = 0;
      __loop_i_18__ = 0;
      __body_i_18__ = 0;
      __loop_i_19__ = 0;
      __body_i_19__ = 0;
      __if_2__ = 0;
      __then_2__ = 0;
      __else_3__ = 0;
      init_write_ghosts(__thread_id__,1);
      communicate_reads(__thread_id__,1);
      for (k = 0; k < __nlocal_1__; ++k) {
        for (i = __lb_i_i_9__[__loop_i_9__]; i < __ub_i_i_9__[__loop_i_9__]; ++i) {
          __o_phi_j_8__offset = __o_phi_j_8_[__loop_i_10__];
	  __trace_phi_f_t_14__offset = __trace_phi_f_t_14__[__loop_i_10__] - __lb_j_i_10__[__loop_i_10__];
          for (j = __lb_j_i_10__[__loop_i_10__]; j < __ub_j_i_10__[__loop_i_10__]; ++j) {
            phi_f_t[__trace_phi_f_t_14__offset+j] = __local_phi__[__o_phi_j_8__offset + j];
            __body_i_10__ = __body_i_10__ + 1;;
          }
          __loop_i_10__ = __loop_i_10__ + 1;
          __body_i_9__ = __body_i_9__ + 1;;
        }
        __loop_i_9__ = __loop_i_9__ + 1;
        for (p = __lb_p_i_11__[__loop_i_11__]; p < __ub_p_i_11__[__loop_i_11__]; ++p) {
          if (__if_cond_2__[__if_2__]) {
            for (i = __lb_i_i_12__[__loop_i_12__]; i < __ub_i_i_12__[__loop_i_12__]; ++i) {
              for (j = __lb_j_i_13__[__loop_i_13__]; j < __ub_j_i_13__[__loop_i_13__]; ++j) {
                phi_f_t[__trace_phi_f_t_15__[__body_i_13__]] = __local_phi__[__i_phi_j_9_[__body_i_13__]];
                phi_f_t[__trace_phi_f_t_16__[__body_i_13__]] = __local_phi__[__i_phi_j_10_[__body_i_13__]];
                phi_f_t[__trace_phi_f_t_17__[__body_i_13__]] = __local_phi__[__i_phi_j_11_[__body_i_13__]];
                phi_f_t[__trace_phi_f_t_18__[__body_i_13__]] = __local_phi__[__i_phi_j_12_[__body_i_13__]];
                __body_i_13__ = __body_i_13__ + 1;;
              }
              __loop_i_13__ = __loop_i_13__ + 1;
              __body_i_12__ = __body_i_12__ + 1;;
            }
            __loop_i_12__ = __loop_i_12__ + 1;
            __then_2__ = __then_2__ + 1;;
          }
          else {
            for (i = __lb_i_i_14__[__loop_i_14__]; i < __ub_i_i_14__[__loop_i_14__]; ++i) {
              for (j = __lb_j_i_15__[__loop_i_15__]; j < __ub_j_i_15__[__loop_i_15__]; ++j) {
                phi_f_t[__trace_phi_f_t_19__[__body_i_15__]] = __local_phi__[__i_phi_j_13_[__body_i_15__]];
                phi_f_t[__trace_phi_f_t_20__[__body_i_15__]] = __local_phi__[__i_phi_j_13_[__body_i_15__]];
                phi_f_t[__trace_phi_f_t_21__[__body_i_15__]] = __local_phi__[__i_phi_j_13_[__body_i_15__]];
                phi_f_t[__trace_phi_f_t_22__[__body_i_15__]] = __local_phi__[__i_phi_j_13_[__body_i_15__]];
                __body_i_15__ = __body_i_15__ + 1;;
              }
              __loop_i_15__ = __loop_i_15__ + 1;
              __body_i_14__ = __body_i_14__ + 1;;
            }
            __loop_i_14__ = __loop_i_14__ + 1;
            __else_3__ = __else_3__ + 1;;
          }
          __if_2__ = __if_2__ + 1;
          __body_i_11__ = __body_i_11__ + 1;;
        }
        __loop_i_11__ = __loop_i_11__ + 1;
        for (i = __lb_i_i_16__[__loop_i_16__]; i < __ub_i_i_16__[__loop_i_16__]; ++i) {
          __o_rho_j_14__offset = __o_rho_j_14_[__loop_i_17__];
	  __trace_residual_0__offset = __trace_residual_0__[__loop_i_17__] - __lb_j_i_17__[__loop_i_17__];
	  __trace_phi_f_t_23__offset = __trace_phi_f_t_23__[__loop_i_17__] - __lb_j_i_17__[__loop_i_17__];
	  __trace_phi_f_t_24__offset = __trace_phi_f_t_24__[__loop_i_17__] - __lb_j_i_17__[__loop_i_17__];
	  __trace_phi_f_t_25__offset = __trace_phi_f_t_25__[__loop_i_17__] - __lb_j_i_17__[__loop_i_17__];
	  __trace_phi_f_t_26__offset = __trace_phi_f_t_26__[__loop_i_17__] - __lb_j_i_17__[__loop_i_17__];
	  __trace_phi_f_t_27__offset = __trace_phi_f_t_27__[__loop_i_17__] - __lb_j_i_17__[__loop_i_17__];
	  for (j = __lb_j_i_17__[__loop_i_17__]; j < __ub_j_i_17__[__loop_i_17__]; ++j) {
	    residual[__trace_residual_0__offset+j] = __local_rho__[__o_rho_j_14__offset + j] - (phi_f_t[__trace_phi_f_t_23__offset+j] + phi_f_t[__trace_phi_f_t_24__offset+j] + phi_f_t[__trace_phi_f_t_25__offset+j] + phi_f_t[__trace_phi_f_t_26__offset+j] - 4 * phi_f_t[__trace_phi_f_t_27__offset+j] ) * 4.0;
            __body_i_17__ = __body_i_17__ + 1;;
          }
          __loop_i_17__ = __loop_i_17__ + 1;
          __body_i_16__ = __body_i_16__ + 1;;
        }
        __loop_i_16__ = __loop_i_16__ + 1;
        for (i = __lb_i_i_18__[__loop_i_18__]; i < __ub_i_i_18__[__loop_i_18__]; ++i) {
          for (j = __lb_j_i_19__[__loop_i_19__]; j < __ub_j_i_19__[__loop_i_19__]; ++j) {
            __local_rho__[__i_rho_write_j_15_[__body_i_19__]] = ( residual[__trace_residual_1__[__body_i_19__]] + residual[__trace_residual_2__[__body_i_19__]] + residual[__trace_residual_3__[__body_i_19__]] + residual[__trace_residual_4__[__body_i_19__]] ) / 4.0;
            __local_phi_old__[__i_phi_old_j_16_[__body_i_19__]] = 0.0;
            __body_i_19__ = __body_i_19__ + 1;;
          }
          __loop_i_19__ = __loop_i_19__ + 1;
          __body_i_18__ = __body_i_18__ + 1;;
        }
        __loop_i_18__ = __loop_i_18__ + 1;;
      }
      communicate_writes(__thread_id__,1);

#pragma part_loop
      __body_2__ = 0;
      __loop_i_20__ = 0;
      __body_i_20__ = 0;
      __loop_i_21__ = 0;
      __body_i_21__ = 0;
      __loop_i_22__ = 0;
      __body_i_22__ = 0;
      __loop_i_23__ = 0;
      __body_i_23__ = 0;
      __loop_i_24__ = 0;
      __body_i_24__ = 0;
      __loop_i_25__ = 0;
      __body_i_25__ = 0;
      __loop_i_26__ = 0;
      __body_i_26__ = 0;
      __loop_i_27__ = 0;
      __body_i_27__ = 0;
      __loop_i_28__ = 0;
      __body_i_28__ = 0;
      __if_4__ = 0;
      __then_4__ = 0;
      __else_5__ = 0;
      init_write_ghosts(__thread_id__,2);
      communicate_reads(__thread_id__,2);
      for (k = 0; k < __nlocal_2__; ++k) {
        for (i = __lb_i_i_20__[__loop_i_20__]; i < __ub_i_i_20__[__loop_i_20__]; ++i) {
          __o_phi_old_j_17__offset = __o_phi_old_j_17_[__loop_i_21__];
	  __trace_phi_c_t_0__offset = __trace_phi_c_t_0__[__loop_i_21__] - __lb_j_i_21__[__loop_i_21__];
          for (j = __lb_j_i_21__[__loop_i_21__]; j < __ub_j_i_21__[__loop_i_21__]; ++j) {
            phi_c_t[__trace_phi_c_t_0__offset+j] = __local_phi_old__[__o_phi_old_j_17__offset + j];
            __body_i_21__ = __body_i_21__ + 1;;
          }
          __loop_i_21__ = __loop_i_21__ + 1;
          __body_i_20__ = __body_i_20__ + 1;;
        }
        __loop_i_20__ = __loop_i_20__ + 1;
        for (p = __lb_p_i_22__[__loop_i_22__]; p < __ub_p_i_22__[__loop_i_22__]; ++p) {
          if (__if_cond_4__[__if_4__]) {
            for (i = __lb_i_i_23__[__loop_i_23__]; i < __ub_i_i_23__[__loop_i_23__]; ++i) {
	      __trace_phi_c_t_1__offset = __trace_phi_c_t_1__[__loop_i_24__] - __lb_j_i_24__[__loop_i_24__];
              for (j = __lb_j_i_24__[__loop_i_24__]; j < __ub_j_i_24__[__loop_i_24__]; ++j) {
	    	phi_c_t[__trace_phi_c_t_1__offset+j] = bdy_value[neighb[p]];
                __body_i_24__ = __body_i_24__ + 1;;
              }
              __loop_i_24__ = __loop_i_24__ + 1;
              __body_i_23__ = __body_i_23__ + 1;;
            }
            __loop_i_23__ = __loop_i_23__ + 1;
            __then_4__ = __then_4__ + 1;;
          }
          else {
            for (i = __lb_i_i_25__[__loop_i_25__]; i < __ub_i_i_25__[__loop_i_25__]; ++i) {
	      __o_phi_old_j_18__offset = __o_phi_old_j_18_[__loop_i_26__];
	      __trace_phi_c_t_2__offset = __trace_phi_c_t_2__[__loop_i_26__] - __lb_j_i_26__[__loop_i_26__];
              for (j = __lb_j_i_26__[__loop_i_26__]; j < __ub_j_i_26__[__loop_i_26__]; ++j) {
		phi_c_t[__trace_phi_c_t_2__offset+j] = __local_phi_old__[__o_phi_old_j_18__offset+j];
                __body_i_26__ = __body_i_26__ + 1;;
              }
              __loop_i_26__ = __loop_i_26__ + 1;
              __body_i_25__ = __body_i_25__ + 1;;
            }
            __loop_i_25__ = __loop_i_25__ + 1;
            __else_5__ = __else_5__ + 1;;
          }
          __if_4__ = __if_4__ + 1;
          __body_i_22__ = __body_i_22__ + 1;;
        }
        __loop_i_22__ = __loop_i_22__ + 1;

        for (i = __lb_i_i_27__[__loop_i_27__]; i < __ub_i_i_27__[__loop_i_27__]; ++i) {
          __o_phi_j_19__offset = __o_phi_j_19_[__loop_i_28__];
          __o_rho_j_20__offset = __o_rho_j_20_[__loop_i_28__];
	  __trace_phi_c_t_3__offset = __trace_phi_c_t_3__[__loop_i_28__] - __lb_j_i_28__[__loop_i_28__];
	  __trace_phi_c_t_4__offset = __trace_phi_c_t_4__[__loop_i_28__] - __lb_j_i_28__[__loop_i_28__];
	  __trace_phi_c_t_5__offset = __trace_phi_c_t_5__[__loop_i_28__] - __lb_j_i_28__[__loop_i_28__];
	  __trace_phi_c_t_6__offset = __trace_phi_c_t_6__[__loop_i_28__] - __lb_j_i_28__[__loop_i_28__];
	  __trace_phi_c_t_7__offset = __trace_phi_c_t_7__[__loop_i_28__] - __lb_j_i_28__[__loop_i_28__];
	  for (j = __lb_j_i_28__[__loop_i_28__]; j < __ub_j_i_28__[__loop_i_28__]; ++j) {
            __local_phi__[__o_phi_j_19__offset + j] = ( phi_c_t[__trace_phi_c_t_3__offset+j] + phi_c_t[__trace_phi_c_t_4__offset+j] + phi_c_t[__trace_phi_c_t_5__offset+j] + phi_c_t[__trace_phi_c_t_6__offset+j] - 4 * phi_c_t[__trace_phi_c_t_7__offset+j] - __local_rho__[__o_rho_j_20__offset + j] ) / 8.0;
            __body_i_28__ = __body_i_28__ + 1;;
          }
          __loop_i_28__ = __loop_i_28__ + 1;
          __body_i_27__ = __body_i_27__ + 1;;
        }
        __loop_i_27__ = __loop_i_27__ + 1;;
      }
      communicate_writes(__thread_id__,2);
      
#pragma part_loop
      __body_3__ = 0;
      __loop_i_29__ = 0;
      __body_i_29__ = 0;
      __loop_i_30__ = 0;
      __body_i_30__ = 0;
      init_write_ghosts(__thread_id__,3);
      communicate_reads(__thread_id__,3);
      for (k = 0; k < __nlocal_3__; ++k) {
        for (i = __lb_i_i_29__[__loop_i_29__]; i < __ub_i_i_29__[__loop_i_29__]; ++i) {
          for (j = __lb_j_i_30__[__loop_i_30__]; j < __ub_j_i_30__[__loop_i_30__]; ++j) {
            __local_phi_old__[__i_phi_old_j_21_[__body_i_30__]] = __local_phi__[__i_phi_j_22_[__body_i_30__]];
            __local_phi_old__[__i_phi_old_j_23_[__body_i_30__]] = __local_phi__[__i_phi_j_22_[__body_i_30__]];
            __local_phi_old__[__i_phi_old_j_24_[__body_i_30__]] = __local_phi__[__i_phi_j_22_[__body_i_30__]];
            __local_phi_old__[__i_phi_old_j_25_[__body_i_30__]] = __local_phi__[__i_phi_j_22_[__body_i_30__]];
            __body_i_30__ = __body_i_30__ + 1;;
          }
          __loop_i_30__ = __loop_i_30__ + 1;
          __body_i_29__ = __body_i_29__ + 1;;
        }
        __loop_i_29__ = __loop_i_29__ + 1;;
      }
      communicate_writes(__thread_id__,3);
    }
    populate_global_arrays(__thread_id__);
    free(__o_phi_j_6_);
    free(__o_phi_j_8_);
    free(__i_phi_j_9_);
    free(__i_phi_j_10_);
    free(__i_phi_j_11_);
    free(__i_phi_j_12_);
    free(__i_phi_j_13_);
    free(__o_phi_j_19_);
    free(__i_phi_j_22_);
    free(__local_phi__);
    free(__o_phi_old_j_0_);
    free(__i_phi_old_j_1_);
    free(__i_phi_old_j_2_);
    free(__i_phi_old_j_3_);
    free(__i_phi_old_j_4_);
    free(__i_phi_old_j_5_);
    free(__i_phi_old_j_16_);
    free(__o_phi_old_j_17_);
    free(__o_phi_old_j_18_);
    free(__i_phi_old_j_21_);
    free(__i_phi_old_j_23_);
    free(__i_phi_old_j_24_);
    free(__i_phi_old_j_25_);
    free(__local_phi_old__);
    free(__o_rho_j_7_);
    free(__o_rho_j_14_);
    free(__o_rho_j_20_);
    free(__local_rho__);
    free(__i_rho_write_j_15_);
    free(__lb_i_i_0__);
    free(__ub_i_i_0__);
    free(__lb_j_i_1__);
    free(__ub_j_i_1__);
    free(__lb_p_i_2__);
    free(__ub_p_i_2__);
    free(__lb_i_i_3__);
    free(__ub_i_i_3__);
    free(__lb_j_i_4__);
    free(__ub_j_i_4__);
    free(__lb_i_i_5__);
    free(__ub_i_i_5__);
    free(__lb_j_i_6__);
    free(__ub_j_i_6__);
    free(__lb_i_i_7__);
    free(__ub_i_i_7__);
    free(__lb_j_i_8__);
    free(__ub_j_i_8__);
    free(__if_cond_0__);
    free(__lb_i_i_9__);
    free(__ub_i_i_9__);
    free(__lb_j_i_10__);
    free(__ub_j_i_10__);
    free(__lb_p_i_11__);
    free(__ub_p_i_11__);
    free(__lb_i_i_12__);
    free(__ub_i_i_12__);
    free(__lb_j_i_13__);
    free(__ub_j_i_13__);
    free(__lb_i_i_14__);
    free(__ub_i_i_14__);
    free(__lb_j_i_15__);
    free(__ub_j_i_15__);
    free(__lb_i_i_16__);
    free(__ub_i_i_16__);
    free(__lb_j_i_17__);
    free(__ub_j_i_17__);
    free(__lb_i_i_18__);
    free(__ub_i_i_18__);
    free(__lb_j_i_19__);
    free(__ub_j_i_19__);
    free(__if_cond_2__);
    free(__lb_i_i_20__);
    free(__ub_i_i_20__);
    free(__lb_j_i_21__);
    free(__ub_j_i_21__);
    free(__lb_p_i_22__);
    free(__ub_p_i_22__);
    free(__lb_i_i_23__);
    free(__ub_i_i_23__);
    free(__lb_j_i_24__);
    free(__ub_j_i_24__);
    free(__lb_i_i_25__);
    free(__ub_i_i_25__);
    free(__lb_j_i_26__);
    free(__ub_j_i_26__);
    free(__lb_i_i_27__);
    free(__ub_i_i_27__);
    free(__lb_j_i_28__);
    free(__ub_j_i_28__);
    free(__if_cond_4__);
    free(__lb_i_i_29__);
    free(__ub_i_i_29__);
    free(__lb_j_i_30__);
    free(__ub_j_i_30__);
    delete_inspector(__thread_id__);;
  }
if( __proc_id__ == 0 )
  print_output(phi,rho,phi_old,nboxes,size_y,size_x);
  free(bdy_value);
  free(coarse_boxes);
  free(fine_boxes);
  free(start_y);
  free(end_y);
  free(start_x);
  free(end_x);
  free(size_y);
  free(size_x);
  free(ftoc);
  free(patches);
  free(patch_start_y);
  free(patch_end_y);
  free(patch_start_x);
  free(patch_end_x);
  free(neighb_level);
  free(neighb);
  free(box_offset);
  free(phi);
  free(phi_old);
  free(rho);
  MPI_Barrier(MPI_COMM_WORLD);
  ARMCI_Finalize();
  MPI_Finalize();
  return 0;
}
